

Application create: #DDEScripting with:
    (#( DynamicDataExchange Scripting)
        collect: [:each | Smalltalk at: each ifAbsent: [
            Application errorPrerequisite: #DDEScripting missing: each]])!

DDEScripting becomeDefault!
Object subclass: #DDEScriptServer
    instanceVariableNames: 'ddeServerManager application topic model globals cache notifier converters debug '
    classVariableNames: 'Current '
    poolDictionaries: 'DdeConstants '!

DDEScripting becomeDefault!
Object subclass: #FormatConverter
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

DDEScripting becomeDefault!
FormatConverter subclass: #RawTextConverter
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

DDEScripting becomeDefault!
FormatConverter subclass: #RTFConverter
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

DDEScripting becomeDefault!
RTFConverter subclass: #RichTextFormatConverter
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

DDEScripting becomeDefault!
RTFConverter subclass: #TokenizedText2RTFConverter
    instanceVariableNames: 'transformations '
    classVariableNames: ''
    poolDictionaries: ''!

DDEScripting becomeDefault!
Application subclass: #DDEScripting
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!


DDEScripting becomeDefault!

!DDEScripting class privateMethods !

abtIsViewApplication

	^true! !

DDEScripting class categoriesFor: #'abtIsViewApplication' are: #(#AbtEdit #- #AutoGen)!

!DDEScriptServer class publicMethods !

default
	"Returns the default (running) instance of the receiver."

	Current isNil ifTrue: [ self initializeCurrent ].
	Current isRunning ifFalse: [ Current startUp ]. "auto startup"
	^Current!

initializeCurrent
	" Private - Initializes the default instance of the receiver."

	Current := self new!

restart
	"Restart the default instance of the receiver."

	Current isNil ifFalse: [ Current shutDown ].
	^self default!

restartDebugging
	"Restart the default instance of the receiver."

	^self restart
		debug: true;
		yourself! !

DDEScriptServer class categoriesFor: #'default' are: #('default server')!
DDEScriptServer class categoriesFor: #'initializeCurrent' are: #('default server')!
DDEScriptServer class categoriesFor: #'restart' are: #('default server')!
DDEScriptServer class categoriesFor: #'restartDebugging' are: #('default server')!

!DDEScriptServer publicMethods !

adaptResult: aResult
	" Private - Adapt aResult to be passed by DDE. "

	(aResult isKindOf: String) ifTrue: [ ^aResult ].
	(aResult isKindOf: Signal) ifTrue: [
		^'RuntimeError: ',
			((aResult arguments notEmpty and: [aResult argument isString])
			ifTrue: [aResult argument]
			ifFalse: [aResult description])
	].
	^aResult printString!

adaptResult: anObject to: format
	" Private - Adapt anObject to be passed by DDE in the format. "

	| converter |
	converter := self converterFor: anObject to: format.
	converter isNil ifTrue: [ ^self adaptResult: anObject ].
	^converter convert: anObject to: format!

application
	" Private - Returns the application name. "

	application isNil ifTrue: [ self initializeApplication ].
	^application!

basicEvaluateNode: parseNode
	" Private - Evaluate parseNode and returns the result of evaluation in the receiver. "

	parseNode isAssociation ifTrue: [ ^parseNode value ].
	(parseNode respondsTo: #evaluate:) ifTrue: [
		^parseNode evaluate: self model
	].
	^parseNode!

cache
	"Returns the receiver's code cache."

	cache isNil ifTrue: [ self initializeCache ].
	^cache!

coldlink: ddeServer clientData: unused callData: ddeCallbackData
	"Coldlink callback - check the item name.
	 Set the returnValue to true if the item is valid."

	| item format valid |
	item := ddeCallbackData item.
	format := ddeCallbackData format.
	valid := self isValidItem: item andFormat: format.

	self notify: '#coldlink: ',item printString,',',format printString,' valid:',valid printString.

	ddeCallbackData returnValue: valid.!

converterFor: anObject to: format
	" Private - Returns the converter to be used to convert anObject to format. "

	^self converters
		detect: [:one| one canConvert: anObject to: format ]
		ifNone: []!

converters
	"Returns the receiver's converters."

	converters isNil ifTrue: [ self initializeConverters ].
	^converters!

debug
	"Returns the receiver's debug flag."

	debug isNil ifTrue: [ debug := false ].
	^debug!

debug: aBoolean
	"Sets the receiver's debug flag."

	debug := aBoolean!

defaultConverters
	" Private - Returns a collection set with default converters. "

	^(Array
		with: RichTextFormatConverter new
		with: RawTextConverter new
	) select: [:each| each isValid ]!

evaluate: aString
	" Private - Perform parsing on aString and evaluate it.
	Returns the result of evaluation. "

	^self parse: aString evaluating: true!

evaluateNode: parseNode
	" Private - Evaluate parseNode and returns the result of evaluation in the receiver. "

	| result |
	self debug ifTrue: [ ^self basicEvaluateNode: parseNode ].
	[
		result := self basicEvaluateNode: parseNode
	] when: ExAll do: [:exception| ^exception ].
	^result!

globals
	"Returns the global objects registered in the receiver."

	globals isNil ifTrue: [ self initializeGlobals ].
	^globals!

hotlink: ddeServer clientData: unused callData: ddeCallbackData
	"Hotlink callback - check the item name and the format.
	 Set the returnValue to true if the item and format are valid."

	| item format valid |
	item := ddeCallbackData item.
	format := ddeCallbackData format.
	valid := self isValidItem: item andFormat: format.

	self notify: '#hotlink: ',item printString,',',format printString,' valid:',valid printString.

	ddeCallbackData returnValue: valid.!

initializeApplication
	" Private - Initializes the application name. "

	application := 'Smalltalk'!

initializeCache
	" Private - Initializes the code cache. "

	cache := Dictionary new!

initializeConverters
	" Private - Initializes the converters collection. "

	converters := OrderedCollection new.
	converters addAll: self defaultConverters.!

initializeDDEServices
	" Private - Initialize the ddeServices. "

	ddeServerManager := DdeServerManager name: self application.

	ddeServerManager
		addCallback: DdeNcoldlinkCallback
		receiver: self
		selector: #coldlink:clientData:callData:
		clientData: nil.

	ddeServerManager
		addCallback: DdeNhotlinkCallback
		receiver: self
		selector: #hotlink:clientData:callData:
		clientData: nil.

	ddeServerManager
		addCallback: DdeNinitiateCallback
		receiver: self
		selector: #initiate:clientData:callData:
		clientData: nil.

	ddeServerManager
		addCallback: DdeNrequestCallback
		receiver: self
		selector: #request:clientData:callData:
		clientData: nil.

	ddeServerManager
		addCallback: DdeNwarmlinkCallback
		receiver: self
		selector: #warmlink:clientData:callData:
		clientData: nil.

	ddeServerManager
		addCallback: DdeNterminationCallback
		receiver: self
		selector: #termination:clientData:callData:
		clientData: nil.!

initializeGlobals
	" Private - Initializes the globals pool. "

	globals := ParseContext in: ParseContext default.
	globals at: 'server' put: self.!

initializeTopic
	" Private - Initializes the topic name. "

	topic := 'Script'!

initiate: aDdeServerManager clientData: unused callData: ddeCallbackData
	"Initiate callback - check the application name and topic.
	 Set the returnValue to true if they match."

	| app aTopic valid |
	app := ddeCallbackData application asLowercase.
	aTopic := ddeCallbackData topic asLowercase.
	valid := self isValidApplication: app andTopic: aTopic.

	self notify: '#initiate: ',app printString,',',aTopic printString,' valid:',valid printString.

	valid ifTrue: [ aDdeServerManager notifyClientOfSupportFor: aTopic ].
	ddeCallbackData returnValue: valid.!

isRunning
	"Returns true when the receiver is running."

	^ddeServerManager notNil!

isValidApplication: app
	" Private - Returns true if application is valid. "

	^app = self application asLowercase!

isValidApplication: app andTopic: aTopic
	" Private - Returns true if application & topic are valid. "

	^(self isValidApplication: app) and: [ self isValidTopic: aTopic ]!

isValidFormat: format
	" Private - Returns true if format is valid. "

	self converters
		detect: [:one| one supports: format]
		ifNone: [ ^false ].
	^true!

isValidItem: item
	" Private - Returns true if item is valid. "

	^true!

isValidItem: item andFormat: format
	" Private - Returns true if item & format are valid. "

	^(self isValidItem: item) and: [ self isValidFormat: format ]!

isValidTopic: aTopic
	" Private - Returns true if topic is valid. "

	^aTopic = self topic asLowercase!

message: aString
	"Notify the user about aString."

	self notify: aString!

model
	"Returns the receiver's model."

	^model!

model: anObject
	"Sets the receiver's model."

	model := anObject.
	self initializeCache.!

notifier
	"Returns the receiver's notifier."

	^notifier!

notifier: aNotifier
	"Sets the receiver's notifier."

	notifier := aNotifier!

notify: aString
	"Notify the user about aString."

	self notifier isNil ifTrue: [ ^self ].

	(self notifier respondsTo: #object:)
	ifTrue: [ ^self notifier object: aString ].

	self notifier cr;nextPutAll: aString.!

parse: aString evaluating: evaluating
	" Private - Perform parsing on aString and evaluate it if evaluating is true.
	Returns the result of evaluation or the parse node (if not evaluated, yet). "

	| node |
	node := self parse: aString onError: [:error| ^error printString ].
	evaluating ifTrue: [ ^self evaluateNode: node ].
	^node!

parse: aString onError: errorBlock
	" Private - Parse aString and returns the root node of the parsing.
	If occurs an error errorBlock will be valued with the cause as an argument."

	| aParser script |
	script := self cache at: aString ifAbsent: [].
	script notNil ifTrue: [ ^script ].
	aParser := self parserFor: aString.
	script := aParser parse: aString onError: errorBlock.
	self cache at: aString put: script.
	^script!

parserFor: aString
	" Private - Returns the parser to be used to parse aString. "

	^ScriptParser for: self model on: self globals!

register: anObject as: aName
	"Register anObject with aName.
	The object can be seen by scripts with a global variable named aName.
	"

	^self globals at: aName put: anObject.!

releaseDDEServices
	" Release the DDEServices resources."

	ddeServerManager notNil ifTrue: [
		ddeServerManager free.
		ddeServerManager := nil.
	].
	self == Current ifTrue: [ Current := nil ].!

request: ddeServer clientData: unused callData: ddeCallbackData
 	"Request callback - Check the item name and the format.
	 If they match what we are looking for then send the data to the client"

	| item format result valid |
	item := ddeCallbackData item.
	format := ddeCallbackData format.
	valid := self isValidItem: item andFormat: format.

	self notify: '#request: ',item printString,',',format printString,' valid:',valid printString.

	valid ifFalse: [
		ddeCallbackData returnValue: false.
		^self
	].

	result := self evaluate: item.
	ddeServer
		sendItem: ddeCallbackData item
		value: (self adaptResult: result to: format)
		format: format.

	ddeCallbackData returnValue: true.!

shutDown
	"ShutDown the receiver as a DDE Server."

	self releaseDDEServices.!

startUp
	"Start/register the receiver as a DDE Server."

	self initializeDDEServices.!

stringFormatName
	" Private - Returns the name used for string transfer format. "

	^'string'!

termination: ddeServer clientData: unused callData: ddeCallbackData
	"Termination callback - Print out a message that the client is disconnecting."

	self message: 'Connection terminated'.
!

topic
	" Private - Returns the topic name. "

	topic isNil ifTrue: [ self initializeTopic ].
	^topic!

warmlink: ddeServer clientData: unused callData: ddeCallbackData
	"Warmlink callback - check the item name and the format.
	 Set the returnValue to true if the item and format are valid."

	| item format valid |
	item := ddeCallbackData item.
	format := ddeCallbackData format.
	valid := self isValidItem: item andFormat: format.

	self notify: '#warmlink: ',item printString,',',format printString,' valid:',valid printString.

	ddeCallbackData returnValue: valid.! !

DDEScriptServer categoriesFor: #'adaptResult:' are: #('DDEServices - Conversions')!
DDEScriptServer categoriesFor: #'adaptResult:to:' are: #('DDEServices - Conversions')!
DDEScriptServer categoriesFor: #'application' are: #('DDEServices - Naming')!
DDEScriptServer categoriesFor: #'basicEvaluateNode:' are: #('DDEServices - Parsing & evaluating')!
DDEScriptServer categoriesFor: #'cache' are: #('accessors')!
DDEScriptServer categoriesFor: #'coldlink:clientData:callData:' are: #('DDEServices - Callbacks')!
DDEScriptServer categoriesFor: #'converterFor:to:' are: #('DDEServices - Conversions')!
DDEScriptServer categoriesFor: #'converters' are: #('accessors')!
DDEScriptServer categoriesFor: #'debug' are: #('accessors')!
DDEScriptServer categoriesFor: #'debug:' are: #('accessors')!
DDEScriptServer categoriesFor: #'defaultConverters' are: #('DDEServices - Initialize/release')!
DDEScriptServer categoriesFor: #'evaluate:' are: #('DDEServices - Parsing & evaluating')!
DDEScriptServer categoriesFor: #'evaluateNode:' are: #('DDEServices - Parsing & evaluating')!
DDEScriptServer categoriesFor: #'globals' are: #('DDEServices - Parsing & evaluating')!
DDEScriptServer categoriesFor: #'hotlink:clientData:callData:' are: #('DDEServices - Callbacks')!
DDEScriptServer categoriesFor: #'initializeApplication' are: #('DDEServices - Initialize/release')!
DDEScriptServer categoriesFor: #'initializeCache' are: #('DDEServices - Initialize/release')!
DDEScriptServer categoriesFor: #'initializeConverters' are: #('DDEServices - Initialize/release')!
DDEScriptServer categoriesFor: #'initializeDDEServices' are: #('DDEServices - Initialize/release')!
DDEScriptServer categoriesFor: #'initializeGlobals' are: #('DDEServices - Initialize/release')!
DDEScriptServer categoriesFor: #'initializeTopic' are: #('DDEServices - Initialize/release')!
DDEScriptServer categoriesFor: #'initiate:clientData:callData:' are: #('DDEServices - Callbacks')!
DDEScriptServer categoriesFor: #'isRunning' are: #('DDEServices -  StartUp')!
DDEScriptServer categoriesFor: #'isValidApplication:' are: #('DDEServices - Validation')!
DDEScriptServer categoriesFor: #'isValidApplication:andTopic:' are: #('DDEServices - Validation')!
DDEScriptServer categoriesFor: #'isValidFormat:' are: #('DDEServices - Validation')!
DDEScriptServer categoriesFor: #'isValidItem:' are: #('DDEServices - Validation')!
DDEScriptServer categoriesFor: #'isValidItem:andFormat:' are: #('DDEServices - Validation')!
DDEScriptServer categoriesFor: #'isValidTopic:' are: #('DDEServices - Validation')!
DDEScriptServer categoriesFor: #'message:' are: #('Notification')!
DDEScriptServer categoriesFor: #'model' are: #('accessors')!
DDEScriptServer categoriesFor: #'model:' are: #('accessors')!
DDEScriptServer categoriesFor: #'notifier' are: #('accessors')!
DDEScriptServer categoriesFor: #'notifier:' are: #('accessors')!
DDEScriptServer categoriesFor: #'notify:' are: #('Notification')!
DDEScriptServer categoriesFor: #'parse:evaluating:' are: #('DDEServices - Parsing & evaluating')!
DDEScriptServer categoriesFor: #'parse:onError:' are: #('DDEServices - Parsing & evaluating')!
DDEScriptServer categoriesFor: #'parserFor:' are: #('DDEServices - Parsing & evaluating')!
DDEScriptServer categoriesFor: #'register:as:' are: #('DDEServices - Parsing & evaluating')!
DDEScriptServer categoriesFor: #'releaseDDEServices' are: #('DDEServices - Initialize/release')!
DDEScriptServer categoriesFor: #'request:clientData:callData:' are: #('DDEServices - Callbacks')!
DDEScriptServer categoriesFor: #'shutDown' are: #('DDEServices -  StartUp')!
DDEScriptServer categoriesFor: #'startUp' are: #('DDEServices -  StartUp')!
DDEScriptServer categoriesFor: #'stringFormatName' are: #('DDEServices - Naming')!
DDEScriptServer categoriesFor: #'termination:clientData:callData:' are: #('DDEServices - Callbacks')!
DDEScriptServer categoriesFor: #'topic' are: #('DDEServices - Naming')!
DDEScriptServer categoriesFor: #'warmlink:clientData:callData:' are: #('DDEServices - Callbacks')!

!FormatConverter class publicMethods !

supportedFormats
	"The receiver must return a collection with all the formats supported.
	This method MUST be implemented by subclasses."

	^self subclassResponsibility! !

FormatConverter class categoriesFor: #'supportedFormats' are: #('format registration')!

!FormatConverter publicMethods !

canConvert: anObject to: format
	"Returns true if the receiver can convert anObject to format.
	Returns true when the receiver can support format.
	This method can be refined by subclasses."

	^self supports: format!

convert: anObject to: format
	"Returns anObject converted to format.
	This method MUST be implemented by subclasses."

	^self subclassResponsibility!

convertToString: anObject
	"Returns anObject converted to raw text format."

	(anObject isKindOf: String) ifTrue: [ ^anObject ].

	(anObject isKindOf: Signal) ifTrue: [
		^'RuntimeError: ',
			((anObject arguments notEmpty and: [anObject argument isString])
			ifTrue: [anObject argument]
			ifFalse: [anObject description])
	].

	(anObject respondsTo: #asString)
	ifTrue: [ ^anObject asString ].

	^anObject printString!

isValid
	"Returns true if the receiver can work at current conditions.
	The default answer is to return true.
	This method can be refined by subclasses."

	^true!

supports: aFormatName
	"Returns true if the receiver can support convertions to/from format named aFormatName.
	By default returns tru if the format is registered in class's supported formats.
	This method can be refined by subclasses. "

	^self species supportedFormats includes: aFormatName! !

FormatConverter categoriesFor: #'canConvert:to:' are: #('query')!
FormatConverter categoriesFor: #'convert:to:' are: #('converting')!
FormatConverter categoriesFor: #'convertToString:' are: #('converting')!
FormatConverter categoriesFor: #'isValid' are: #('query')!
FormatConverter categoriesFor: #'supports:' are: #('query')!

!RawTextConverter class publicMethods !

supportedFormats
	"The receiver must return a collection with all the formats supported."

	^Array with: 'string'! !

RawTextConverter class categoriesFor: #'supportedFormats' are: #('format registration')!

!RawTextConverter publicMethods !

convert: anObject to: format
	"Returns anObject converted to format."

	^self convertToString: anObject! !

RawTextConverter categoriesFor: #'convert:to:' are: #('converting')!

!RichTextFormatConverter class publicMethods !

supportedFormats
	"The receiver must return a collection with all the formats supported."

	^Array with: 'rich text format' with: 'rtf'! !

RichTextFormatConverter class categoriesFor: #'supportedFormats' are: #('format registration')!

!RichTextFormatConverter publicMethods !

arrayToRTF: aCollection
	" Private - Returns aCollection converted to RTF format."

	| stream width height table column |
	(self canBeTable: aCollection) ifFalse: [
		^self basicConvertToRTF: aCollection
	].

	width := aCollection size.
	height := 0. aCollection do: [:each| height := height max: each size ].
	table := self rtfTableSupport width: width height: height.
	1 to: width do: [:x|
		column := aCollection at: x.
		1 to: column size do: [:y|
			(table cellAt: x@y) fillWith: (column at: y)
		]
	].
	^table!

basicConvertToRTF: anObject
	" Private - Returns anObject converted to RTF format.
	This is the basic implementation.
	This method can be refined by subclasses."

	| string |
	(anObject respondsTo: #asRTF) ifTrue: [
		(anObject isKindOf: self rtfFileSupport) ifTrue: [ ^anObject ].
		^self rtfFileWith: anObject
	].
	string := self convertToString: anObject.
	^self rtfFileWith: (self rtfTextSupport from: string)!

canBeTable: aCollection
	" Private - Returns true if aCollection can be converted to an RTF table."

	self rtfTableSupport isNil ifTrue: [ ^false ].
	aCollection do: [:each|
		(self isArray: each) ifFalse: [ ^false ].
	].
	^true
	!

convert: anObject to: format
	"Returns anObject converted to format."

	^(self convertToRTF: anObject) asByteArray!

convertToRTF: anObject
	" Private - Returns anObject converted to RTF format."

	^self basicConvertToRTF: 
		((self isArray: anObject)
		ifTrue: [ self arrayToRTF: anObject ]
		ifFalse: [ anObject ])!

isArray: anObject
	" Private - Returns true if anObject is considered an array. "

	^(anObject isKindOf: String) not
	and: [ anObject isKindOf: SequenceableCollection ]!

rtfFileWith: anRTFElement
	" Private - Returns an RTF File with document containing anRTFElement."

	| rtf |
	rtf := self rtfFileSupport new.
	rtf document section text add: anRTFElement.
	^rtf! !

RichTextFormatConverter categoriesFor: #'arrayToRTF:' are: #('converting')!
RichTextFormatConverter categoriesFor: #'basicConvertToRTF:' are: #('converting')!
RichTextFormatConverter categoriesFor: #'canBeTable:' are: #('converting')!
RichTextFormatConverter categoriesFor: #'convert:to:' are: #('converting')!
RichTextFormatConverter categoriesFor: #'convertToRTF:' are: #('converting')!
RichTextFormatConverter categoriesFor: #'isArray:' are: #('converting')!
RichTextFormatConverter categoriesFor: #'rtfFileWith:' are: #('converting')!

!RTFConverter publicMethods !

isValid
	"Returns true if the receiver can work at current conditions."

	^self rtfSupportExists!

rtfBoldSupport
	" Private - Returns the class to support RTF Bold style. "

	^Smalltalk at: #RTFBold!

rtfFileSupport
	" Private - Returns the class to support RTF files.
	Generates an error if there is not support loaded in this image. "

	^Smalltalk at: #RTFFile!

rtfSupportExists
	" Private - Returns true if RTF support is found. "

	Smalltalk at: #RTFElement ifAbsent: [ ^false ].
	^true!

rtfTableSupport
	" Private - Returns the class to support RTF Tables (or nil). "

	^Smalltalk at: #RTFTableModel ifAbsent: []!

rtfTextSupport
	" Private - Returns the class to support RTF raw Text.
	Generates an error if there is not support loaded in this image. "

	^Smalltalk at: #RTFText!

rtfUnderlineSupport
	" Private - Returns the class to support RTF Underline style. "

	^Smalltalk at: #RTFUnderline! !

RTFConverter categoriesFor: #'isValid' are: #('query')!
RTFConverter categoriesFor: #'rtfBoldSupport' are: #('RTF builders')!
RTFConverter categoriesFor: #'rtfFileSupport' are: #('RTF builders')!
RTFConverter categoriesFor: #'rtfSupportExists' are: #('RTF builders')!
RTFConverter categoriesFor: #'rtfTableSupport' are: #('RTF builders')!
RTFConverter categoriesFor: #'rtfTextSupport' are: #('RTF builders')!
RTFConverter categoriesFor: #'rtfUnderlineSupport' are: #('RTF builders')!

!TokenizedText2RTFConverter class publicMethods !

supportedFormats
	"The receiver must return a collection with all the formats supported."

	^Array with: 'tokenized text'! !

TokenizedText2RTFConverter class categoriesFor: #'supportedFormats' are: #('format registration')!

!TokenizedText2RTFConverter publicMethods !

convert: anObject
	"Returns anObject converted to format."

	^self
		convert: anObject
		to: self species supportedFormats first!

convert: anObject to: format
	"Returns anObject converted to format."

	^self convertToRTF: (self convertToString: anObject)!

convertToRTF: aString
	" Private - Returns a RTFParagraphText element composed
	with text in aString performing token transformations. "

	^self
		rtfTextFrom: aString
		replacedWith: (self replacementsFor: aString)!

initializeTransformations
	" Private - Initializes the receiver's transformations. "

	transformations := Dictionary new.!

replacementsFor: aString
	" Private - Returns a collection of replacements to be done on aString. "

	| replacements start index stream |
	replacements := SortedCollection sortBlock: [:a :b | a key <= b key ].
	self transformations keys do: [:token|
		start := 1.
		[ index := aString indexOfSubCollection: token startingAt: start.
		  index >= start ] whileTrue: [
		  replacements add: (Association key: index value: token).
		  start := index + token size.
		].
	].
	^replacements!

rtfTextFrom: aString replacedWith: replacements
	" Private - Returns a text element obtained replacing with replacements. "

	| result start |
	result := self rtfTextSupport new.
	start := 1.
	replacements do: [:assoc|
		assoc key > start ifTrue: [ result := result , (aString copyFrom: start to: assoc key - 1) ].
		result := result , (self transformations at: assoc value).
		start := assoc key + assoc value size
	].
	^result , (aString copyFrom: start to: aString size)

"
(HalconStrokes2RTFConverter new convert: 'hola Ale!!!! que?? tal!!') asRTF
"!

transform: aToken to: transformation
	"Register aToken to be transformed into transformation.
	transformation can be aRTFElement or aBlock (evaluable to aRTFElement). "

	self transformations
		at: aToken
		put: transformation!

transformations
	" Returns the receivers transformations dictionary. "

	transformations isNil ifTrue: [ self initializeTransformations ].
	^transformations! !

TokenizedText2RTFConverter categoriesFor: #'convert:' are: #('converting')!
TokenizedText2RTFConverter categoriesFor: #'convert:to:' are: #('converting')!
TokenizedText2RTFConverter categoriesFor: #'convertToRTF:' are: #('converting')!
TokenizedText2RTFConverter categoriesFor: #'initializeTransformations' are: #('initialization')!
TokenizedText2RTFConverter categoriesFor: #'replacementsFor:' are: #('converting')!
TokenizedText2RTFConverter categoriesFor: #'rtfTextFrom:replacedWith:' are: #('converting')!
TokenizedText2RTFConverter categoriesFor: #'transform:to:' are: #('transformations')!
TokenizedText2RTFConverter categoriesFor: #'transformations' are: #('accessing')!

DDEScriptServer initializeAfterLoad!
FormatConverter initializeAfterLoad!
RawTextConverter initializeAfterLoad!
RTFConverter initializeAfterLoad!
RichTextFormatConverter initializeAfterLoad!
TokenizedText2RTFConverter initializeAfterLoad!
DDEScripting initializeAfterLoad!

DDEScripting loaded!