

Application create: #Scripting with:
    (#( AbtViewApplication)
        collect: [:each | Smalltalk at: each ifAbsent: [
            Application errorPrerequisite: #Scripting missing: each]])!

Scripting becomeDefault!
Object subclass: #ParseBlock
    instanceVariableNames: 'receiver parseNode '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
Object subclass: #ParseContext
    instanceVariableNames: 'map parent '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
Object subclass: #ParseNode
    instanceVariableNames: 'sourceRange '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseNode subclass: #AssignNode
    instanceVariableNames: 'variable expression '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseNode subclass: #MessageNode
    instanceVariableNames: 'receiver selector arguments '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseNode subclass: #ParseStatements
    instanceVariableNames: 'temporaries statements '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseStatements subclass: #BlockNode
    instanceVariableNames: 'arguments '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseStatements subclass: #CascadeNode
    instanceVariableNames: 'receiver '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseStatements subclass: #Script
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseNode subclass: #ParseVariable
    instanceVariableNames: 'binding '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseVariable subclass: #ParseArgument
    instanceVariableNames: 'canBeAssigned '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseVariable subclass: #ParseGlobalVariable
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseGlobalVariable subclass: #ParseGlobalConstant
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseVariable subclass: #ParseLiteral
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseVariable subclass: #ParseSpecialVariable
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseSpecialVariable subclass: #ParseInstanceVariable
    instanceVariableNames: 'name index '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseSpecialVariable subclass: #ParseSelf
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseNode subclass: #ReturnNode
    instanceVariableNames: 'expression exitBlock '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
Object subclass: #ScriptParser
    instanceVariableNames: 'source scope instanceVariables errorBlock '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
Object subclass: #ServiceError
    instanceVariableNames: 'description '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ServiceError subclass: #ParseError
    instanceVariableNames: 'source position '
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
ParseError subclass: #SyntaxError
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!

Scripting becomeDefault!
Application subclass: #Scripting
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!


Scripting becomeDefault!

!AssignNode publicMethods !

evaluate: aReceiver
	"Must evaluate the receiver using aditional information provided by aReceiver."

	^self variable bind: (self expression evaluate: aReceiver) to: aReceiver!

exitBlock: aBlock
	"Sets the exit block of the receiver.
	This block will be used to abort secuential execution of the receiver."

	super exitBlock: aBlock.
	self variable exitBlock: aBlock.
	self expression exitBlock: aBlock.!

expression
	"Returns the receiver's expression."

	^expression!

expression: anExpression
	"Sets the receiver's expression."

	expression := anExpression!

variable
	"Returns the receiver's variable."

	^variable!

variable: aVariable
	"Sets the receiver's variable."

	variable := aVariable! !

AssignNode categoriesFor: #'evaluate:' are: #('evaluation')!
AssignNode categoriesFor: #'exitBlock:' are: #('evaluation')!
AssignNode categoriesFor: #'expression' are: #('accessors')!
AssignNode categoriesFor: #'expression:' are: #('accessors')!
AssignNode categoriesFor: #'variable' are: #('accessors')!
AssignNode categoriesFor: #'variable:' are: #('accessors')!

!BlockNode publicMethods !

activateContext
	" Private - Activate the receiver's context. "

	super activateContext.
	self clear: self arguments.!

arguments
	"Returns the receiver's arguments."

	arguments isNil ifTrue: [ self initializeArguments ].
	^arguments!

arguments: aCollection
	"Set the receiver's arguments."

	arguments := aCollection!

evaluate: aReceiver
	"Must evaluate the receiver using aditional information provided by aReceiver.
	Block defer evaluation until message #value:... is sent to them.
	This method acts as method context activation."

	self activateContext.
	^ParseBlock new
		parseNode: self;
		receiver: aReceiver;
		yourself!

exitBlock: aBlock
	"Sets the exit block of the receiver.
	This block will be used to abort secuential execution of the receiver."

	super exitBlock: aBlock.
	self arguments do: [:each| each exitBlock: aBlock ].!

initializeArguments
	" Private - Initializes the receiver's arguments."

	arguments := OrderedCollection new.! !

BlockNode categoriesFor: #'activateContext' are: #('evaluation')!
BlockNode categoriesFor: #'arguments' are: #('accessors')!
BlockNode categoriesFor: #'arguments:' are: #('accessors')!
BlockNode categoriesFor: #'evaluate:' are: #('evaluation')!
BlockNode categoriesFor: #'exitBlock:' are: #('evaluation')!
BlockNode categoriesFor: #'initializeArguments' are: #('accessors')!

!CascadeNode publicMethods !

receiver
	" Returns the receiver's receiver."

	^receiver!

receiver: aReceiver
	" Set the receiver's receiver."

	receiver := aReceiver! !

CascadeNode categoriesFor: #'receiver' are: #('accessors')!
CascadeNode categoriesFor: #'receiver:' are: #('accessors')!

!MessageNode class publicMethods !

receiver: receiver selector: selector arguments: arguments
	"Returns a new instance of the receiver."

	^self new
		receiver: receiver;
		selector: selector;
		arguments: arguments;
		yourself! !

MessageNode class categoriesFor: #'receiver:selector:arguments:' are: #('instance creation')!

!MessageNode publicMethods !

arguments
	^arguments!

arguments: aCollection
	arguments := aCollection!

evaluate: aReceiver
	"Must evaluate the receiver using aditional information provided by aReceiver."

	^(self receiver evaluate: aReceiver)
		perform: self selector "value"
		withArguments: (self arguments collect: [:each| each evaluate: aReceiver ])!

exitBlock: aBlock
	"Sets the exit block of the receiver.
	This block will be used to abort secuential execution of the receiver."

	super exitBlock: aBlock.
	self receiver exitBlock: aBlock.
	self arguments do: [:each| each exitBlock: aBlock ].!

receiver
	^receiver!

receiver: aParseNode
	receiver := aParseNode!

selector
	^selector!

selector: aSymbol
	selector := aSymbol! !

MessageNode categoriesFor: #'arguments' are: #('accessors')!
MessageNode categoriesFor: #'arguments:' are: #('accessors')!
MessageNode categoriesFor: #'evaluate:' are: #('evaluation')!
MessageNode categoriesFor: #'exitBlock:' are: #('evaluation')!
MessageNode categoriesFor: #'receiver' are: #('accessors')!
MessageNode categoriesFor: #'receiver:' are: #('accessors')!
MessageNode categoriesFor: #'selector' are: #('accessors')!
MessageNode categoriesFor: #'selector:' are: #('accessors')!

!ParseArgument publicMethods !

canBeAssigned
	"Returns true if the receiver is a variable that can be assigned."

	^canBeAssigned == true!

canBeAssigned: aBoolean
	"Set canBeAssigned to aBoolean."

	canBeAssigned := aBoolean.!

clear
	"Clear the receiver's contents."

	^self binding value: nil! !

ParseArgument categoriesFor: #'canBeAssigned' are: #('query')!
ParseArgument categoriesFor: #'canBeAssigned:' are: #('accessors')!
ParseArgument categoriesFor: #'clear' are: #('evaluation')!

!ParseBlock publicMethods !

abtWhileFalse
		"Repetitively evaluate the receiver block and aBlock,
		 until the result of receiver block evaluation is
		 true.  Answer nil."
	^self whileFalse: []!

abtWhileTrue
		"Repetitively evaluate the receiver block and aBlock,
		 until the result of receiver block evaluation is
		 false.  Answer nil."
	^self whileTrue: []!

apply: aCollection from: start to: end
	"Evaluate the receiver for each variable slot of aCollection from start to end.  Answer aCollection."

	start to: end do: [:i | self value: (aCollection basicAt: i) ].
	^aCollection!

applyWithIndex: aCollection from: start to: end
	"Evaluate the receiver for each variable slot and index of aCollection from start to end.  Answer aCollection."

	start to: end do: [:i | self value: (aCollection basicAt: i) value: i ].
	^aCollection!

applyWithIndex: aCollection from: start to: end offset: offset
	"Evaluate the receiver for each variable slot and index of aCollection from start to end.  Answer aCollection.
	 offset (an Integer) will be added to each index passed to the receiver."

	start to: end do: [:i | self value: (aCollection basicAt: i) value: i + offset ].
	^aCollection!

argumentCount
	"Answer the number of arguments to the receiver."

	^self arguments size!

arguments
	" Returns the receiver's arguments. "

	^parseNode arguments!

atEndOrWhenExceptionDo: completionBlock
	"Execute the completionBlock if any exception occurs during the evaluation of the receiver. A 
	 futher exception handler is looked for after the completion block is executed. The 
	 completionBlock is also executed after successful completion of the receiver."

	^[ self value ] atEndOrWhenExceptionDo: completionBlock!

bind: arguments to: anArray
	" Private - Bind the arguments in aCollection to the real arguments in anArray. "

	| stream |
	arguments size = 0 ifTrue: [ ^self ].
	arguments size = anArray size ifFalse: [
		^self error: 'Invalid number of arguments.'
	].

	stream := ReadStream on: anArray.
	arguments do: [:each| each setValueTo: stream next ].!

ensure: terminateBlock
	"Evaluate the receiver and return its result. Evaluate terminateBlock when the receiver returns normally or is prematurely terminated."

	^[ self value ] ensure: terminateBlock!

fork
	"Create a new Process which is scheduled by the
	 ProcessScheduler. The new process executes the receiver by
	 sending it the message value. The new process is created
	 with the same prority as the activeProcess. Answer the
	 receiver."


	^[ self value ] fork!

forkAt: aPriority
	"Create a new Process which is scheduled by the
	 ProcessScheduler. The new process executes the receiver by
	 sending it the message value. The new process is created
	 with priority aPriority. Answer the receiver."

	^[ self value ] forkAt: aPriority!

forkAt: aPriority named: processName

	"Create a new Process called processName which is scheduled by the
	ProcessorScheduler. The new process executes the receiver by sending 
	it the message value. The new process is created with priority aPriority. 
	Answer the newly created process."

	^[ self value ] forkAt: aPriority named: processName!

forkNamed: processName

	"Create a new Process named processName which is scheduled by the
	ProcessScheduler. The new process executes the receiver by sending it 
	the message value. The new process is created with the same prority as 
	the activeProcess. Answer the new process."

	^[ self value ] forkNamed: processName!

ifCurtailed: terminateBlock 
	"Evaluate the receiver and return its result. Evaluate terminateBlock when the receiver  is prematurely terminated."

	| earlyReturn |
	earlyReturn := true.
	^[ | returnValue |
		returnValue := self value.
	 	earlyReturn := false.
		returnValue
	] valueOnReturnDo: [earlyReturn ifTrue: [terminateBlock value]].!

newProcess
	"Create a new Process which is in suspended state by the
	 ProcessScheduler. The new process executes the receiver by
	 sending it the message value. The new process is created
	 with the same prority as the activeProcess. Answer the
	 newly created process."

	^[ self value ] newProcess!

parseNode: aNode
	" Sets the receiver's parent parseNode. "

	parseNode := aNode!

receiver: aReceiver
	" Sets the receiver's receiver for evaluation. "

	receiver := aReceiver!

statements
	" Returns the receiver's statements. "

	^parseNode statements!

value
	"Evaluate the receiver."

	^self valueWithArguments: #()!

value: argument
	"Evaluate the receiver."

	^self valueWithArguments: (Array with: argument)!

value: anObject value: anotherObject
	"Evaluate the receiver."

	^self valueWithArguments: (Array with: anObject with: anotherObject)!

value: anObject value: anotherObject value: thirdObject
	"Evaluate the receiver."

	^self valueWithArguments: (Array with: anObject with: anotherObject with: thirdObject)!

valueWithArguments: anArray
	"Evaluate the receiver with parameters anArray.
	Answer the result of the last statement to be evaluated."

	| result |
	self bind: self arguments to: anArray.
	self statements do: [:each| result := each evaluate: receiver ].
	^result!

when: exception do: handlerBlock
	"Evaluate the receiver. If the exception or a more general exception is
	 signaled during execution of the receiver, evaluate the handlerBlock."

	^[ self value ] when: exception do: handlerBlock!

when: exception1 do: block1 when: exception2 do: block2
	"Evaluate the receiver. If an exception is signaled during execution
	 of the receiver, if one of the exceptions is the exception that was
	 signaled or a more general exception, evaluate the corresponding
	 block. The exceptions are tested in order."

	^[ self value ] when: exception1 do: block1 when: exception2 do: block2!

when: exception1 do: block1 when: exception2 do: block2 when: exception3 do: block3
	"Evaluate the receiver. If an exception is signaled during execution
	 of the receiver, if one of the exceptions is the exception that was
	 signaled or a more general exception, evaluate the corresponding
	 block. The exceptions are tested in order."

	^[ self value ] when: exception1 do: block1 when: exception2 do: block2 when: exception3 do: block3!

when: exception1 do: block1 when: exception2 do: block2 when: exception3 do: block3
	when: exception4 do: block4

	"Evaluate the receiver. If an exception is signaled during execution
	 of the receiver, if one of the exceptions is the exception that was
	 signaled or a more general exception, evaluate the corresponding
	 block. The exceptions are tested in order."

	^[ self value ]
		when: exception1 do: block1
		when: exception2 do: block2
		when: exception3 do: block3
		when: exception4 do: block4!

when: exception1 do: block1 when: exception2 do: block2 when: exception3 do: block3
	when: exception4 do: block4 when: exception5 do: block5

	"Evaluate the receiver. If an exception is signaled during execution
	 of the receiver, if one of the exceptions is the exception that was
	 signaled or a more general exception, evaluate the corresponding
	 block. The exceptions are tested in order."

	^[ self value ]
		when: exception1 do: block1
		when: exception2 do: block2
		when: exception3 do: block3
		when: exception4 do: block4
		when: exception5 do: block5!

whenExceptionDo: completionBlock
	"Execute the completionBlock if any exception occurs during the evaluation of the receiver. A 
	 futher exception handler is looked for after the completion block is executed."

	^[ self value ] whenExceptionDo: completionBlock!

whileFalse: aBlock

	"Evaluate the receiver and aBlock repeatedly until the receiver evaluates
	 to true."

	[ self value ] whileFalse: [ aBlock value ]
!

whileTrue: aBlock

	"Evaluate the receiver and aBlock repeatedly until the receiver evaluates
	 to false."

	[ self value ] whileTrue: [ aBlock value ]
! !

ParseBlock categoriesFor: #'abtWhileFalse' are: #('block emulation: AbtRun-API')!
ParseBlock categoriesFor: #'abtWhileTrue' are: #('block emulation: AbtRun-API')!
ParseBlock categoriesFor: #'apply:from:to:' are: #('block emulation: ES-Internal')!
ParseBlock categoriesFor: #'applyWithIndex:from:to:' are: #('block emulation: ES-Internal')!
ParseBlock categoriesFor: #'applyWithIndex:from:to:offset:' are: #('block emulation: ES-Internal')!
ParseBlock categoriesFor: #'argumentCount' are: #('accessors')!
ParseBlock categoriesFor: #'arguments' are: #('accessors')!
ParseBlock categoriesFor: #'atEndOrWhenExceptionDo:' are: #('block emulation: CLDT-API')!
ParseBlock categoriesFor: #'bind:to:' are: #('dynamic binding')!
ParseBlock categoriesFor: #'ensure:' are: #('block emulation: CLDT-API')!
ParseBlock categoriesFor: #'fork' are: #('block emulation: CPM-API')!
ParseBlock categoriesFor: #'forkAt:' are: #('block emulation: CPM-API')!
ParseBlock categoriesFor: #'forkAt:named:' are: #('block emulation: ES-Internal')!
ParseBlock categoriesFor: #'forkNamed:' are: #('block emulation: ES-Internal')!
ParseBlock categoriesFor: #'ifCurtailed:' are: #('block emulation: CLDT-API')!
ParseBlock categoriesFor: #'newProcess' are: #('block emulation: CPM-API')!
ParseBlock categoriesFor: #'parseNode:' are: #('accessors')!
ParseBlock categoriesFor: #'receiver:' are: #('accessors')!
ParseBlock categoriesFor: #'statements' are: #('accessors')!
ParseBlock categoriesFor: #'value' are: #('evaluation')!
ParseBlock categoriesFor: #'value:' are: #('evaluation')!
ParseBlock categoriesFor: #'value:value:' are: #('evaluation')!
ParseBlock categoriesFor: #'value:value:value:' are: #('evaluation')!
ParseBlock categoriesFor: #'valueWithArguments:' are: #('evaluation')!
ParseBlock categoriesFor: #'when:do:' are: #('block emulation: CLDT-API')!
ParseBlock categoriesFor: #'when:do:when:do:' are: #('block emulation: CLDT-API')!
ParseBlock categoriesFor: #'when:do:when:do:when:do:' are: #('block emulation: CLDT-API')!
ParseBlock categoriesFor: #'when:do:when:do:when:do:when:do:' are: #('block emulation: CLDT-API')!
ParseBlock categoriesFor: #'when:do:when:do:when:do:when:do:when:do:' are: #('block emulation: CLDT-API')!
ParseBlock categoriesFor: #'whenExceptionDo:' are: #('block emulation: CLDT-API')!
ParseBlock categoriesFor: #'whileFalse:' are: #('block emulation: CLDT-API')!
ParseBlock categoriesFor: #'whileTrue:' are: #('block emulation: CLDT-API')!

!ParseContext class publicMethods !

default
	"Returns the default scope."

	^self new
		beDefault;
		yourself!

in: aParentScope
	"Returns the a new scope included in aParentScope."

	^self new
		parent: aParentScope;
		yourself! !

ParseContext class categoriesFor: #'default' are: #('instance creation')!
ParseContext class categoriesFor: #'in:' are: #('instance creation')!

!ParseContext publicMethods !

associationAt: aName
	" LookUp for variable named aName in the receiver.
	Returns the association found for aName or produces an error if failed. "

	^self associationAt: aName ifAbsent: [ self error: 'Absent element.' ]!

associationAt: aName ifAbsent: errorBlock
	" LookUp for variable named aName in the receiver.
	Returns the association found for aName or evaluates errorBlock if failed. "

	| assoc key |
	aName size = 0 ifTrue: [ ^nil ].

	assoc := self associationIn: self map at: aName ifAbsent: [].
	assoc notNil ifTrue: [ ^assoc ].

	(parent isKindOf: KeyedCollection) ifTrue: [
		assoc := self associationIn: parent at: aName ifAbsent: [].
		assoc notNil ifTrue: [ ^assoc ].
	].

	^parent notNil
	ifTrue: [ parent associationAt: aName ifAbsent: errorBlock ]
	ifFalse: [ errorBlock value ]!

associationIn: aDictionary at: aName ifAbsent: errorBlock
	" Private - LookUp for variable named aName in the dictionary aDictionary. "

	| assoc key |
	^aDictionary associationAt: aName ifAbsent: [
		key := aDictionary keys detect: [:one| one isSymbol and: [one asString = aName]] ifNone: [].
		key notNil ifTrue: [ aDictionary associationAt: key ifAbsent: [] ]
	]!

at: aKey put: anObject
	"Creates a new variable binding for aKey in the receiver's local context (the map)."

	^self map at: aKey put: anObject!

beDefault
	" Private - Set the receiver with default (SYSTEM) settings. "

	parent := Smalltalk.!

initializeMap
	" Private - Initializes the receiver's map. "

	map := Dictionary new.!

map
	" Private - Returns the receiver's map."

	map isNil ifTrue: [ self initializeMap ].
	^map
	!

parent: aScope
	" Private - Sets the receiver's parent scope."

	parent := aScope
	! !

ParseContext categoriesFor: #'associationAt:' are: #('lookup')!
ParseContext categoriesFor: #'associationAt:ifAbsent:' are: #('lookup')!
ParseContext categoriesFor: #'associationIn:at:ifAbsent:' are: #('lookup')!
ParseContext categoriesFor: #'at:put:' are: #('bindings')!
ParseContext categoriesFor: #'beDefault' are: #('initialization')!
ParseContext categoriesFor: #'initializeMap' are: #('initialization')!
ParseContext categoriesFor: #'map' are: #('accessors')!
ParseContext categoriesFor: #'parent:' are: #('accessors')!

!ParseError publicMethods !

descriptionString
	"Returns the receivers description as shown to the user."

	| stream |
	self source isNil ifTrue: [ ^super descriptionString ].
	stream := ReadStream on: self source.
	^' ',(stream next: (self position min: stream size))
	,'<---',self description,' ',(stream upTo: nil)!

position
	"Returns the receiver's position."

	^position!

position: index
	"Set the receiver's position."

	position := index!

source
	"Returns the receiver's source."

	^source!

source: aString
	"Set the receiver's source."

	source := aString! !

ParseError categoriesFor: #'descriptionString' are: #('accessors')!
ParseError categoriesFor: #'position' are: #('accessors')!
ParseError categoriesFor: #'position:' are: #('accessors')!
ParseError categoriesFor: #'source' are: #('accessors')!
ParseError categoriesFor: #'source:' are: #('accessors')!

!ParseGlobalConstant publicMethods !

canBeAssigned
	"Returns true if the receiver is a variable that can be assigned."

	^false! !

ParseGlobalConstant categoriesFor: #'canBeAssigned' are: #('query')!

!ParseGlobalVariable publicMethods !

association: anAssoc
	"Sets the receiver's association."

	self binding: anAssoc!

canBeAssigned
	"Returns true if the receiver is a variable that can be assigned."

	^true! !

ParseGlobalVariable categoriesFor: #'association:' are: #('accessors')!
ParseGlobalVariable categoriesFor: #'canBeAssigned' are: #('query')!

!ParseInstanceVariable class publicMethods !

name: aName index: index
	"Returns an instance of the receiver with name aName."

	^self new
		name: aName;
		index: index;
		yourself! !

ParseInstanceVariable class categoriesFor: #'name:index:' are: #('instance creation')!

!ParseInstanceVariable publicMethods !

bind: aValue to: anObject
	"Set the receiver's value to aValue.
	Check if the receiver is assignable.
	anObject is the receiver of the asignation context.
	WARNING: The receiver MUST return the set value."

	self canBeAssigned ifFalse: [
		^self error: 'Invalid assignment.'
	].
	^anObject
		instVarAt: self index
		put: aValue!

canBeAssigned
	"Returns true if the receiver is a variable that can be assigned."

	^true!

evaluate: aReceiver
	"Must evaluate the receiver using aditional information provided by aReceiver."

	^aReceiver instVarAt: self index!

index
	"Returns the receiver's index."

	^index!

index: anIndex
	"Sets the receiver's index."

	index := anIndex!

name
	"Returns the receiver's name."

	^name!

name: aName
	"Sets the receiver's name."

	name := aName! !

ParseInstanceVariable categoriesFor: #'bind:to:' are: #('evaluation')!
ParseInstanceVariable categoriesFor: #'canBeAssigned' are: #('query')!
ParseInstanceVariable categoriesFor: #'evaluate:' are: #('evaluation')!
ParseInstanceVariable categoriesFor: #'index' are: #('accessors')!
ParseInstanceVariable categoriesFor: #'index:' are: #('accessors')!
ParseInstanceVariable categoriesFor: #'name' are: #('accessors')!
ParseInstanceVariable categoriesFor: #'name:' are: #('accessors')!

!ParseLiteral class publicMethods !

from: start to: stop value: aValue
	"Returns an instance of teh receiver with value aValue."

	^(self from: start to: stop)
		binding: (Association key: #literal value: aValue);
		yourself!

value: aValue
	"Returns an instance of teh receiver with value aValue."

	^self new
		binding: (Association key: #literal value: aValue);
		yourself! !

ParseLiteral class categoriesFor: #'from:to:value:' are: #('instance creation')!
ParseLiteral class categoriesFor: #'value:' are: #('instance creation')!

!ParseNode class publicMethods !

from: start to: stop
	"Returns an instance of the receiver."

	^self new
		sourceRange: (start to: stop);
		yourself!

sourceRange: interval
	"Returns an instance of the receiver."

	^self new
		sourceRange: interval;
		yourself! !

ParseNode class categoriesFor: #'from:to:' are: #('instance creation')!
ParseNode class categoriesFor: #'sourceRange:' are: #('instance creation')!

!ParseNode publicMethods !

, aParseNode
	"Returns a composite parse node with the receiver and aParseNode appended at end."

	^ParseStatements
		with: self
		with: aParseNode!

clear
	" Private - Clear the receiver's cached state.
	This method must be implemented by subclasses."

	^self subclassResponsibility.!

evaluate
	"Evaluate the receiver in an empty context."

	^self evaluate: nil!

evaluate: aReceiver
	"Must evaluate the receiver using aditional information provided by aReceiver.
	This message must be implemented by the subclasses."

	^self subclassResponsibility!

exitBlock: aBlock
	"Sets the exit block of the receiver.
	This block will be used to abort secuential execution of the receiver.
	Do nothing by default.
	This method can be refined by subclasses."!

isComposite
	"Returns true if the receiver is a composite parse node."

	^false!

isReturn
	"Returns true if the receiver is a return parse node."

	^false!

isSuper
	"Returns true if the receiver is a reference to super."

	^false!

isVariable
	"Returns true if the receiver is a variable parse node."

	^false!

sourcePosition: anInterval
	"Sets the receiver's sourceRange."

	^self sourceRange: anInterval!

sourceRange
	"Returns the receiver's sourceRange."

	^sourceRange!

sourceRange: anInterval
	"Sets the receiver's sourceRange."

	sourceRange := anInterval! !

ParseNode categoriesFor: #',' are: #('operations')!
ParseNode categoriesFor: #'clear' are: #('evaluation')!
ParseNode categoriesFor: #'evaluate' are: #('evaluation')!
ParseNode categoriesFor: #'evaluate:' are: #('evaluation')!
ParseNode categoriesFor: #'exitBlock:' are: #('evaluation')!
ParseNode categoriesFor: #'isComposite' are: #('query')!
ParseNode categoriesFor: #'isReturn' are: #('query')!
ParseNode categoriesFor: #'isSuper' are: #('query')!
ParseNode categoriesFor: #'isVariable' are: #('query')!
ParseNode categoriesFor: #'sourcePosition:' are: #('source management')!
ParseNode categoriesFor: #'sourceRange' are: #('source management')!
ParseNode categoriesFor: #'sourceRange:' are: #('source management')!

!ParseSelf publicMethods !

evaluate: aReceiver
	"Must evaluate the receiver using aditional information provided by aReceiver."

	^aReceiver! !

ParseSelf categoriesFor: #'evaluate:' are: #('evaluation')!

!ParseStatements class publicMethods !

with: aStatement
	"Returns an instance of the receiver containing aStatement."

	^self new
		add: aStatement;
		yourself!

with: statement1 with: statement2
	"Returns an instance of the receiver containing the statements."

	^self new
		add: statement1;
		add: statement2;
		yourself!

with: statement1 with: statement2 with: statement3
	"Returns an instance of the receiver containing the statements."

	^self new
		add: statement1;
		add: statement2;
		add: statement3;
		yourself!

with: statement1 with: statement2 with: statement3 with: statement4
	"Returns an instance of the receiver containing the statements."

	^self new
		add: statement1;
		add: statement2;
		add: statement3;
		add: statement4;
		yourself!

withAll: aCollection
	"Returns an instance of the receiver containing all the statements in aCollection."

	^self new
		addAll: aCollection;
		yourself! !

ParseStatements class categoriesFor: #'with:' are: #('instance creation')!
ParseStatements class categoriesFor: #'with:with:' are: #('instance creation')!
ParseStatements class categoriesFor: #'with:with:with:' are: #('instance creation')!
ParseStatements class categoriesFor: #'with:with:with:with:' are: #('instance creation')!
ParseStatements class categoriesFor: #'withAll:' are: #('instance creation')!

!ParseStatements publicMethods !

activateContext
	" Private - Activate the receiver's context. "

	self
		clear: self temporaries;
		yourself.!

add: aStatement
	"Add aStatement to the receiver."

	^self addLast: aStatement!

addAll: aCollection
	"Add aCollection of statements to the receiver."

	^self statements addAll: aCollection!

addFirst: aStatement
	"Add aStatement to the receiver."

	^self statements addFirst: aStatement!

addFirstAll: aCollection
	"Add aCollection of statements to the receiver."

	aCollection asArray reverseDo: [:each|
		self statements addFirst: each
	]!

addLast: aStatement
	"Add aStatement to the receiver."

	^self statements addLast: aStatement!

clear: aCollection
	" Private - Clear the values in aCollection. "

	aCollection size = 0 ifTrue: [ ^self ].
	aCollection do: [:each| each clear ].!

evaluate: aReceiver
	"Must evaluate the receiver using aditional information provided by aReceiver."

	| result |
	self activateContext.
	self statements do: [:each| result := each evaluate: aReceiver ].
	^result!

exitBlock: aBlock
	"Sets the exit block of the receiver.
	This block will be used to abort secuential execution of the receiver."

	super exitBlock: aBlock.
	self temporaries do: [:each| each exitBlock: aBlock ].
	self statements do: [:each| each exitBlock: aBlock ].!

initializeStatements
	" Private - Initialize the receiver's statements. "

	statements := OrderedCollection new.!

initializeTemporaries
	" Private - Initialize the receiver's temporaries. "

	temporaries := OrderedCollection new.!

isComposite
	"Returns true if the receiver is a composite parse node."

	^true!

isEmpty
	"Returns true if the receiver is empty."

	^self statements isEmpty!

remove: aStatement
	"Remove aStatement from the receiver."

	^self statements remove: aStatement!

removeAll: aCollection
	"Remove aCollection of statements from the receiver."

	^self statements removeAll: aCollection!

statements
	"Returns the receiver's statements."

	statements isNil ifTrue: [ self initializeStatements ].
	^statements!

temporaries
	"Returns the receiver's temporaries."

	temporaries isNil ifTrue: [ self initializeTemporaries ].
	^temporaries!

temporaries: aCollection
	"Sets the receiver's temporaries."

	temporaries := aCollection! !

ParseStatements categoriesFor: #'activateContext' are: #('evaluation')!
ParseStatements categoriesFor: #'add:' are: #('elements management')!
ParseStatements categoriesFor: #'addAll:' are: #('elements management')!
ParseStatements categoriesFor: #'addFirst:' are: #('elements management')!
ParseStatements categoriesFor: #'addFirstAll:' are: #('elements management')!
ParseStatements categoriesFor: #'addLast:' are: #('elements management')!
ParseStatements categoriesFor: #'clear:' are: #('evaluation')!
ParseStatements categoriesFor: #'evaluate:' are: #('evaluation')!
ParseStatements categoriesFor: #'exitBlock:' are: #('evaluation')!
ParseStatements categoriesFor: #'initializeStatements' are: #('initialization')!
ParseStatements categoriesFor: #'initializeTemporaries' are: #('initialization')!
ParseStatements categoriesFor: #'isComposite' are: #('query')!
ParseStatements categoriesFor: #'isEmpty' are: #('query')!
ParseStatements categoriesFor: #'remove:' are: #('elements management')!
ParseStatements categoriesFor: #'removeAll:' are: #('elements management')!
ParseStatements categoriesFor: #'statements' are: #('accessors')!
ParseStatements categoriesFor: #'temporaries' are: #('accessors')!
ParseStatements categoriesFor: #'temporaries:' are: #('accessors')!

!ParseVariable publicMethods !

bind: aValue to: anObject
	"Set the receiver's value to aValue.
	Check if the receiver is assignable.
	anObject is the receiver of the asignation context.
	WARNING: The receiver MUST return the set value."

	self canBeAssigned ifFalse: [
		^self error: 'Invalid assignment.'
	].
	^self binding value: aValue!

binding
	"Returns the receiver's binding."

	^binding!

binding: anAssociation
	"Sets the receiver's binding."

	binding := anAssociation!

canBeAssigned
	"Returns true if the receiver is a variable that can be assigned."

	^false!

evaluate: aReceiver
	"Must evaluate the receiver using aditional information provided by aReceiver."

	^self binding value!

isVariable
	"Returns true if the receiver is a variable parse node."

	^true!

name
	"Returns the receiver's name."

	^self binding key!

setValueTo: anObject
	"Set the receiver's value to anObject."

	^self binding value: anObject.! !

ParseVariable categoriesFor: #'bind:to:' are: #('evaluation')!
ParseVariable categoriesFor: #'binding' are: #('accessors')!
ParseVariable categoriesFor: #'binding:' are: #('accessors')!
ParseVariable categoriesFor: #'canBeAssigned' are: #('query')!
ParseVariable categoriesFor: #'evaluate:' are: #('evaluation')!
ParseVariable categoriesFor: #'isVariable' are: #('query')!
ParseVariable categoriesFor: #'name' are: #('accessors')!
ParseVariable categoriesFor: #'setValueTo:' are: #('evaluation')!

!ReturnNode publicMethods !

evaluate: aReceiver
	"Must evaluate the receiver using aditional information provided by aReceiver."

	exitBlock isNil ifTrue: [ ^self error: 'Invalid script context.' ].
	^exitBlock value: (self expression evaluate: aReceiver)!

exitBlock
	"Returns the receiver's exitBlock.
	This block will be needed during evaluation of the receiver to perform script termination."

	^exitBlock!

exitBlock: aBlock
	"Sets the receiver's exitBlock.
	This block will be needed during evaluation of the receiver to perform script termination."

	super exitBlock: aBlock.
	exitBlock := aBlock.

	expression exitBlock: aBlock.!

expression
	"Returns the receiver's expression."

	^expression!

expression: anExpressionNode
	"Sets the receiver's expression."

	expression := anExpressionNode.!

isReturn
	"Returns true if the receiver is a return parse node."

	^true! !

ReturnNode categoriesFor: #'evaluate:' are: #('evaluation')!
ReturnNode categoriesFor: #'exitBlock' are: #('accessors')!
ReturnNode categoriesFor: #'exitBlock:' are: #('accessors')!
ReturnNode categoriesFor: #'expression' are: #('accessors')!
ReturnNode categoriesFor: #'expression:' are: #('accessors')!
ReturnNode categoriesFor: #'isReturn' are: #('query')!

!Script publicMethods !

clearExitBlock
	"Clear the exit block."

	self exitBlock: nil.!

evaluate: aReceiver
	"Must evaluate the receiver using aditional information provided by aReceiver."

	self exitBlock: [:result| self clearExitBlock. ^result ].
	^super evaluate: aReceiver! !

Script categoriesFor: #'clearExitBlock' are: #('Private - exitBlock')!
Script categoriesFor: #'evaluate:' are: #('evaluation')!

!Scripting class privateMethods !

abtIsViewApplication

	^true! !

Scripting class categoriesFor: #'abtIsViewApplication' are: #(#AbtEdit #- #AutoGen)!

!ScriptParser class publicMethods !

for: aReceiver on: scope
	"Returns an instance of the receiver seen objects at scope."

	^self new
		scope: scope;
		receiver: aReceiver;
		yourself!

on: scope
	"Returns an instance of the receiver seen objects at scope."

	^self new
		scope: scope;
		yourself! !

ScriptParser class categoriesFor: #'for:on:' are: #('instance creation')!
ScriptParser class categoriesFor: #'on:' are: #('instance creation')!

!ScriptParser publicMethods !

arrayLiteralElement
	" Private - Gets the next array element from the receiver."

	^self constant!

assignment: aVariable in: aContext
	" Private - Get the an assignement from the receiver.
	<rule> expression = {variableName ':='} (primary | messageExpression {';' cascadeMessage} ). 
	 "

	| expression |
	aVariable canBeAssigned ifFalse: [ ^self expected: 'Can''t assign to this variable. Assign' ].
	expression := self expression: aContext.
	expression isNil ifTrue: [ ^self expected: 'Expression' ].
	^AssignNode new 
		variable: aVariable;
		expression: expression;
		yourself!

assignment: variable in: aContext startPosition: startPosition
	" Private - Get the an assignement from the receiver. "

	| assignment |
	assignment := self assignment: variable in: aContext.
	assignment sourceRange: (startPosition to: self position).
	^assignment!

atEnd
	"Returns true when the receiver is atEnd."

	self skipSeparators.
	^source atEnd!

basicPeek
	"Peek the next character from source stream."

	source peek = $" ifTrue: [ "skip coments"
		source next;upTo: $"
	].
	^source peek!

braceExpression: aContext
	" Private - Parse the receiver looking for an expression inside brackets using variables in aContext.
	Returns nil if failed. "

	^self notFullyImplemented: #braceExpression:!

cascade: expression to: aReceiver in: aContext
	" Private - Parse the receiver looking for a cascade expression using variables in aContext.
	<rule> cascadeMessage = unaryMessage | binaryMessage | keywordMessage. 
	"

	| start cascade message |
	start := self position.
	cascade := CascadeNode with: expression.
	[	message := self message3: aReceiver repeat: false in: aContext.
		message isNil
		ifTrue: [ self expected: 'Message expression' ]
		ifFalse: [	cascade add: message ].
		self match: self semicolon ] whileTrue: [].
	cascade sourceRange: (start to: self position).
	^cascade!

checkForOldSyntax: radix with: value on: aStream
	" Private - Answer aNumber as described on the stream, aStream."

	(aStream atEnd not and: [aStream peekFor: $.])
	ifTrue: [
		(aStream atEnd not and: [aStream peek digitValue between: 0 and: radix-1])
		ifTrue: [ ^self syntaxError: 'Non-radix 10 numbers may not contain decimal points' ]
		ifFalse: [ aStream skip: -1 ]
	].
	(aStream atEnd not and: [aStream peek asLowercase = $e])
	ifTrue: [ ^self syntaxError: 'Non-radix 10 numbers may not have an exponent' ].
	^value!

chooseFloatRepresentationFor: exponentChar

	(('deqs' includes: exponentChar)
			or: [true"SystemUtils isVW20LanguageCompatible"
			and: [exponentChar = $f]])
		ifFalse:	[^nil].
	exponentChar == $e "Float exponentCharacter" ifTrue:	[^Float].
"
	(exponentChar == Double exponentCharacter
			or: [exponentChar == Double altExponentCharacter])
		ifTrue:	[^Double].
	(exponentChar == FixedPoint exponentCharacter
			or: [exponentChar == $f])
		ifTrue:	[^FixedPoint].
"
	^self error: 'This exponent character has not been adequately supported'!

colon
	" Private - Returns the colon token. "

	^$:!

constant
	" Private - Recognize a constant (named, number, character,string, or quoted).
	Returns the constant value."

	self skipSeparators.
	(self match: 'false') ifTrue: [ ^false].
	(self match: 'true') ifTrue: [ ^true].
	(self match: 'nil') ifTrue: [ ^nil].
	(self match: self singleQuote) ifTrue: [ ^self nextString ].
	(self peekAny: '0123456789-' advance: false) notNil ifTrue: [ ^self nextNumber ].
	(self match: self dollar) ifTrue: [ ^self nextCharacter ].
	(self match: (Array with: self literalQuote with: self leftParenthesis)) ifTrue: [ ^self nextArray ].
	(self match: self literalQuote) ifTrue: [ ^self nextSelector asSymbol ].
	(self match: self leftParenthesis) ifTrue: [ ^self nextArray ].
	(self match: self leftBracket) ifTrue: [ ^self nextByteArray ].
	" (self match: self leftBrace) ifTrue: [ ^self nextQualifiedNameLiteral ]. "

	^self nextSelector asSymbol
	" ^self expected: 'word, binary, keyword, (, or ['  "!

dollar
	" Private - Returns the dollar token. "

	^$$!

expected: aString
	"An expected element can't be found."

	| syntaxError |
	syntaxError := SyntaxError description: aString ,' expected.'.
	syntaxError
		source: source contents;
		position: self position + 1;
		yourself.
	^errorBlock value: syntaxError!

expression: aContext
	" Private - Parse the receiver looking for an expression using variables in aContext. Returns nil if failed.
	<rule> expression = {variableName ':='} (primary | messageExpression {';' cascadeMessage}. 
	"

	| variable expression aReceiver start |
	start := self startOfNextToken.

	variable := self nextVariable: aContext.
	(variable notNil and: [ self match: self leftArrow ])
	ifTrue: [ ^self assignment: variable in: aContext startPosition: start ].

	self position: start.
	aReceiver := self primaryExpression: aContext.
	self atEnd ifTrue: [ ^aReceiver ].
	expression := self message3: aReceiver repeat: true in: aContext.
	expression isNil ifTrue: [ ^aReceiver ].
	(self match: self semicolon) ifTrue: [
		expression := self cascade: expression to: aReceiver in: aContext
	].
	^expression!

initializeInstanceVariables
	" Private - Initializes the receiver's instanceVariables. "

	self instanceVariables: Array new.!

initializeScope
	" Private - Initializes the receiver's scope. "

	self scope: ParseContext default!

instanceVariables
	"Returns the receiver's instanceVariables."

	instanceVariables isNil ifTrue: [ self initializeInstanceVariables ].
	^instanceVariables!

instanceVariables: aCollection
	"Sets the receiver's instanceVariables."

	instanceVariables := aCollection!

isBinarySelector: aSelector
	" Private - Returns true if aSelector is a keyword selector. "

	aSelector isEmpty ifTrue: [ ^false ].
	aSelector
		detect: [:one| (self isSelectorCharacter: one) not ]
		ifNone: [ ^true ].
	^false!

isKeywordSelector: aSelector
	" Private - Returns true if aSelector is a keyword selector. "

	^aSelector last = $:!

isSelectorCharacter: aCharacter
	" Private - Returns true if aCharacter is a selector character.
	<rule> selectorCharacter = ',' | '+' | '/' | '\' | '*' | '~' | '>' | '<' | '=' | '@' | '%' | '|' | '&' | '?' | '!!' | '-'.
	"

	^',+-/\*~><=@%|&?!!' includes: aCharacter!

isUnarySelector: aSelector
	" Private - Returns true if aSelector is a unary selector. "

	aSelector isEmpty ifTrue: [ ^false ].
	aSelector first isLetter ifFalse: [ ^false ].
	aSelector
		detect: [:one| one isAlphaNumeric not ]
		ifNone: [ ^true ].
	^false!

leftArrow
	" Private - Returns the leftArrow token. "

	^':='!

leftBrace
	" Private - Returns the leftBrace token. "

	^${!

leftBracket
	" Private - Returns the leftBracket token. "

	^$[!

leftParenthesis
	" Private - Returns the leftParentesis token. "

	^$(!

literalQuote
	" Private - Returns the literal quote token. "

	^$#!

match: token
	"Try to match token.  token can be a single token or a collection of tokens.
	Returns true if success and advance the receiver's stream.
	If failed, restore the receiver's position."

	| position |
	self atEnd ifTrue: [^false].
	position := self position.
	token isCharacter
	ifTrue: [
		self peek = token
		ifTrue: [ self next.^true ]
	]
	ifFalse: [
		(token isKindOf: Block)
		ifTrue: [
			token argumentCount = 0
			ifTrue: [ token value ifTrue: [ ^true ] ]
			ifFalse: [ (token value: self next) ifTrue: [ ^true ] ]
		]
		ifFalse: [
			token
				detect: [:one| (self match: one) not ]
				ifNone: [ ^true ].
		].
	].
	self position: position.
	^false.!

message1: aNode repeat: repeat in: aContext
	" Private - Parse the receiver looking for a message expression using variables in aContext.
	<rule> messageExpression = unaryExpression | binaryExpression | keywordExpression. 
	<rule> unaryExpression = primaryMessage {unaryMessage}.
	<rule> binaryExpression = (unaryExpression | primary) binaryMessage {binaryMessage}.
	<rule> keywordExpression = (binaryExpression | primary) keywordMessage. 
	"

	| aReceiver start end selector arguments argument part partStart |
	aReceiver := aNode.
	[
		start := self startOfNextToken.
		selector := self nextSelectorOrNil.
		selector isNil ifTrue: [ ^aReceiver ].
		(self isUnarySelector: selector) "simple message"
		ifTrue: [ arguments := #( ) ]
		ifFalse: [
			aReceiver isSuper ifTrue: [ ^self expected: 'message' ].
			self position: start.
			^aReceiver "ouch..."
		].
	aReceiver isNil ifTrue: [ self expected: 'Invalid receiver or invalid name. Variable' ].
	aReceiver := (MessageNode from: start to: end)
		receiver: aReceiver;
		selector: (self newSelectorName: selector);
		arguments: arguments.
	repeat ] whileTrue: [].
	^aReceiver!

message2: aNode repeat: repeat in: aContext
	" Private - Parse the receiver looking for a message expression using variables in aContext.
	<rule> messageExpression = unaryExpression | binaryExpression | keywordExpression. 
	<rule> unaryExpression = primaryMessage {unaryMessage}.
	<rule> binaryExpression = (unaryExpression | primary) binaryMessage {binaryMessage}.
	<rule> keywordExpression = (binaryExpression | primary) keywordMessage. 
	"

	| aReceiver start end selector arguments argument part partStart |
	aReceiver := aNode.
	[
		start := self startOfNextToken.
		selector := self nextSelectorOrNil.
		selector isNil ifTrue: [ ^aReceiver ].
		(selector = #| or: [self isBinarySelector: selector]) "level>=2"
		ifTrue: [
			argument := self primaryExpression: aContext.
			argument isNil ifTrue: [ ^self expected: 'argument' ].
			argument := self message1: argument repeat: true in: aContext.
			argument isNil ifTrue: [ ^self expected: 'argument' ].
			arguments := Array with: argument.					
		]
		ifFalse: [
			(self isUnarySelector: selector) "simple message"
			ifTrue: [ arguments := #( ) ]
			ifFalse: [
				aReceiver isSuper ifTrue: [ ^self expected: 'message' ].
				self position: start.
				^aReceiver
			]
		].
	aReceiver isNil ifTrue: [ self expected: 'Invalid receiver or invalid name. Variable' ].
	aReceiver := (MessageNode from: start to: end)
		receiver: aReceiver;
		selector: (self newSelectorName: selector);
		arguments: arguments.
	repeat ] whileTrue: [].
	^aReceiver!

message3: aNode repeat: repeat in: aContext
	" Private - Parse the receiver looking for a message expression using variables in aContext.
	<rule> messageExpression = unaryExpression | binaryExpression | keywordExpression. 
	<rule> unaryExpression = primaryMessage {unaryMessage}.
	<rule> binaryExpression = (unaryExpression | primary) binaryMessage {binaryMessage}.
	<rule> keywordExpression = (binaryExpression | primary) keywordMessage. 
	"

	| aReceiver start end selector arguments argument part partStart |
	aReceiver := aNode.
	[
		start := self startOfNextToken.
		selector := self nextSelectorOrNil.
		selector isNil ifTrue: [ ^aReceiver ].
		(self isKeywordSelector: selector) "level>=3"
		ifTrue: [
			argument := self primaryExpression: aContext.
			argument isNil ifTrue: [ ^self expected: 'argument' ].
			argument := self message2: argument repeat: true in: aContext.
			argument isNil ifTrue: [ ^self expected: 'argument' ].
			arguments := Array with: argument.
			[ partStart := self startOfNextToken.
			part := self nextSelectorOrNil.
			part notNil and: [ self isKeywordSelector: part ]
			] whileTrue: [
				selector := selector , part.
				argument := self primaryExpression: aContext.
				argument isNil ifTrue: [ ^self expected: 'argument' ].
				argument := self message2: argument repeat: true in: aContext.
				argument isNil ifTrue: [ ^self expected: 'argument' ].
				arguments := arguments copyWith: argument.
			].
			self position: partStart.
		]
		ifFalse: [
			(selector = #| or: [self isBinarySelector: selector]) "level>=2"
			ifTrue: [
				argument := self primaryExpression: aContext.
				argument isNil ifTrue: [ ^self expected: 'argument' ].
				argument := self message1: argument repeat: true in: aContext.
				argument isNil ifTrue: [ ^self expected: 'argument' ].
				arguments := Array with: argument.					
			]
			ifFalse: [
				(self isUnarySelector: selector) "simple message"
				ifTrue: [ arguments := #( ) ]
				ifFalse: [
					aReceiver isSuper ifTrue: [ ^self expected: 'message' ].
					self position: start.
					^aReceiver
				]
			]
		].
	aReceiver isNil ifTrue: [ self expected: 'Invalid receiver or invalid name. Variable' ].
	aReceiver := (MessageNode from: start to: end)
		receiver: aReceiver;
		selector: (self newSelectorName: selector);
		arguments: arguments.
	repeat ] whileTrue: [].
	^aReceiver!

methodScope
	" Private - Returns a context for script/method.
	<rule> script = [ | temporaries ] statements. 
	"

	| start newScope |
	(self match: self verticalBar) ifFalse: [ ^self scope ].
	newScope := ParseContext in: self scope.
	self temporaries: newScope.
	^newScope!

mustHaveA: aCharacter
	" Private - Check that the next character is aCharacter.
	Read the character if success. Generates a syntax error in absent."

	self peek = aCharacter ifFalse: [
		^self syntaxError: aCharacter printString,' spected.'
	].
	self atEnd ifFalse: [ self next ].!

newSelectorName: aSelectorName
	" Private - Build a new message name from aSelectorName. "

	^aSelectorName asSymbol "build a symbol without prompting for new symbols."!

next
	"Advance to the next character onto source stream."

	self skipSeparators.
	^source next!

nextArgument: aContext
	" Private - Parse the receiver looking for a new argument using variables in aContext.
	Returns nil if failed. "

	| start word assoc |
	start := self startOfNextToken.
	word := self nextWord.
	word isNil ifTrue: [ ^self expected: 'Temporary/Argument' ].
	aContext at: word put: nil.
	assoc := aContext associationAt: word.
	^(ParseArgument
		from: start to: self position)
		binding: assoc;
		yourself!

nextArray
	" Private - Parse the receiver looking for an Array using variables in aContext.
	Returns anArray or generates an error if failed.
	<rule> array = '(' {number | string | symbol | array | characterConstant} ')'.
	"

	| stream |
	stream := WriteStream on: (Array new: 16).
	[self match: self rightParenthesis]	whileFalse: 	[ stream nextPut: self arrayLiteralElement ].
	^stream contents!

nextArrayConstant: aContext
	" Private - Parse the receiver looking for an Array using variables in aContext.
	Returns nil if failed.
	<rule> arrayConstant = '#' array.
	<rule> array = '(' {number | string | symbol | array | characterConstant} ')'.
	"

	| start value |
	start := self startOfNextToken.
	(self match: (Array with: self literalQuote with: self leftParenthesis)) ifFalse: [ ^nil ].
	value := self nextArray.
	value isNil ifTrue: [ self position: start. ^nil ].
	^ParseLiteral
		from: start to: self position
		value: value!

nextBlock: aContext
	" Private - Parse the receiver looking for a block using variables in aContext.
	Returns nil if failed.
	<node> $[ ($: var)* temporaries statements $] => BlockNode
	"

	| start argNodes block peek |
	start := self position - 1.
	argNodes := #().
	[	self match: self colon	"gather any arguments"
	]	whileTrue: [
		argNodes := argNodes copyWith: (self nextArgument: aContext)
	].
	argNodes size > 0 ifTrue: [
		peek := self peek.
		(peek = self rightBracket or: [ peek = self verticalBar ])
		ifFalse: [ ^self expected: 'Vertical bar' ].
		peek = self verticalBar ifTrue: [ self skip: self verticalBar ].
	].
	block := self statementsArgs: argNodes temps: #() in: aContext.
	(self match: self rightBracket)
	ifFalse: [ ^self expected: 'Period or right bracket' ].
	block sourcePosition: (start to: self position).
	^block!

nextByteArray
	" Private - Parse the receiver looking for aByteArray using variables in aContext.
	Returns aByteArray or generates an error if failed.
	"

	^self notImplementedYet: #nextByteArray!

nextCharacter
	" Private - Read aCharacter from the receiver."

	^source atEnd ifTrue: [ $  ] ifFalse: [ source next ]!

nextCharacterConstant: aContext
	" Private - Parse the receiver looking for aCharacter using variables in aContext.
	Returns nil if failed. "

	| start |
	start := self startOfNextToken.
	(self match: self dollar) ifFalse: [ ^nil ].
	^ParseLiteral from: start to: self position value: self nextCharacter!

nextLiteral: aContext
	" Private - Parse the receiver looking for a literal using variables in aContext.
	Returns nil if failed.
	<rule> literal = number | string | characterConstant | symbolConstant | arrayConstant.
	"

	| node |
	node := self nextNumberConstant: aContext.
	node notNil ifTrue: [ ^node ].
	node := self nextStringConstant: aContext.
	node notNil ifTrue: [ ^node ].
	node := self nextCharacterConstant: aContext.
	node notNil ifTrue: [ ^node ].
	node := self nextArrayConstant: aContext.
	node notNil ifTrue: [ ^node ].
	node := self nextSymbolConstant: aContext.
	node notNil ifTrue: [ ^node ].
	^nil!

nextNumber
	" Private - Parse the receiver looking for a number using variables in aContext.
	Returns aNumber or generates an error if failed.
	<rule> number = [digits 'r'] ['-'] bigDigits ['.'bigDigits] ['e' ['-'] digits].
	<rule> digits = digit {digit}.
	<rule> Digit = '0' | '1' | '2' | '3' | '4' | '5' | '6' | '7' | '8' | '9'.
	<rule> bigDigits = bigDigit {bigDigit}.
	<rule> bigDigit = digit | capitalLetter.
	"

	^self readSmalltalkNumberFrom: source!

nextNumberConstant: aContext
	" Private - Parse the receiver looking for a number using variables in aContext.
	Returns nil if failed.
	<rule> number = [digits 'r'] ['-'] bigDigits ['.'bigDigits] ['e' ['-'] digits].
	<rule> digits = digit {digit}.
	<rule> Digit = '0' | '1' | '2' | '3' | '4' | '5' | '6' | '7' | '8' | '9'.
	<rule> bigDigits = bigDigit {bigDigit}.
	<rule> bigDigit = digit | capitalLetter.
	"

	| start value |
	start := self startOfNextToken.
	(self peekAny: '0123456789-' advance: false) isNil ifTrue: [ ^nil ].
	value := self nextNumber.
	value isNil ifTrue: [ self position: start. ^nil ].
	^ParseLiteral from: start to: self position value: value!

nextSelector
	" Private - Returns the next selector in the receiver.
	Generates an error if failed.
		<rule> symbol = unarySelector | binarySelector | keyword {keyword}.
		<rule> unarySelector = identifier.
		<rule> binarySelector = selectorCharacter [selectorCharacter] | '-'.
		<rule> keyword = identifier ':'.
	"

	| result |
	result := self nextSelectorOrNil.
	result isNil ifTrue: [ ^self expected: 'Symbol' ].
	^result!

nextSelectorOrNil
	" Private - Returns the next selector in the receiver.
	Returns nil if failed.
		<rule> symbol = unarySelector | binarySelector | keyword {keyword}.
		<rule> unarySelector = identifier.
		<rule> binarySelector = selectorCharacter [selectorCharacter] | '-'.
		<rule> keyword = identifier ':'.
	"

	| contents peek |
	peek := self basicPeek.
	(peek isNil or: [ peek isSeparator ]) ifTrue: [ ^nil ].

	(peek = $- or: [ self isSelectorCharacter: peek ]) "binarySelector"
	ifTrue: [ self next. ^String with: peek ].

	peek isLetter ifFalse: [ ^nil ].
	contents := String new.
	[	contents := contents, self nextWord.
		self basicPeek = self colon ] whileTrue: [
		contents := contents,(String with: self colon).
		self skip: self colon.
		peek := self basicPeek.
		(peek isNil or: [ peek isAlphaNumeric not ]) ifTrue: [ ^contents ]
	].
	^contents asSymbol!

nextString
	" Private - Read aStriong from the receiver. "

	| contents |
	contents := String new.
	[	contents := contents , (source upTo: self singleQuote).
		source peek = self singleQuote ] whileTrue: [ source next ].
	^contents!

nextStringConstant: aContext
	" Private - Parse the receiver looking for a string using variables in aContext.
	Returns nil if failed. "

	| start contents |
	start := self startOfNextToken.
	(self match: self singleQuote) ifFalse: [ ^nil ].
	^ParseLiteral from: start to: self position value: self nextString!

nextSymbolConstant: aContext
	" Private - Parse the receiver looking for aSymbol using variables in aContext.
	Returns nil if failed.
		<rule> symbolConstant = '#' symbol.
		<rule> symbol = unarySelector | binarySelector | keyword {keyword}.
		<rule> unarySelector = identifier.
		<rule> binarySelector = selectorCharacter [selectorCharacter] | '-'.
		<rule> keyword = identifier ':'.
	"

	| start value |
	start := self startOfNextToken.
	(self match: self literalQuote) ifFalse: [ ^nil ].
	value := self nextSelectorOrNil.
	value isNil ifTrue: [ self position: start. ^nil ].
	^ParseLiteral
		from: start to: self position
		value: value asSymbol!

nextVariable: aContext
	" Private - Parse the receiver looking for a variable name.
	Returns the variable binding or nil if failed. "

	| word assoc start variable |
	start := self startOfNextToken.
	word := self nextWord.
	word isNil ifTrue: [ ^nil ].

	variable := self specialVariableNamed: word.
	variable notNil ifTrue: [
		variable sourceRange: (start to: self position).
		^variable
	].

	assoc := aContext
		associationAt: word
		ifAbsent: [ self position: start. ^nil ].
	^(ParseGlobalVariable
		from: start to: self position)
		association: assoc;
		yourself!

nextWord
	"Returns the next word in the receiver or nil if failed.
	 Advance the receiver."

	| start word |
	self atEnd ifTrue: [ ^nil ].
	start := self startOfNextToken.
	self peek isLetter ifFalse: [ ^nil ].

	word := String with: self next.
	[	source atEnd not
		and: [ source peek isAlphaNumeric ]
	] whileTrue: [ word := word , (String with: source next) ].

	^word!

notFullyImplemented: aString
	"A Syntax error will be set because the service is not fully implemented yet."

	^self syntaxError: aString,' not implemented yet.'!

parse: aString onError: aBlock
	"Parse aString and returns the root node of the parsing.
	If occurs an error aBlock will be valued with the cause as an argument.
	Example:

	ScriptParser new parse: 'Smalltalk keys' onError: [:error| ^error printString ].

	"

	| script |
	errorBlock := aBlock.
	source := ReadStream on: aString.
	script := Script with: (self statements: self methodScope innerBlock: false).
	errorBlock := source := nil.
	^script!

peek
	"Peek the next character from source stream."

	self skipSeparators.
	^self basicPeek!

peekAny: aCollection
	"Try to match the next token of the receiver to any of the elements in aCollection.
	Returns the first element in aCollection that matches or nil if none."

	^self peekAny: aCollection advance: true!

peekAny: aCollection advance: advance
	"Try to match the next token of the receiver to any of the elements in aCollection.
	Returns the first element in aCollection that matches or nil if none."

	| start |
	start := self position.
	aCollection do: [:each|
		(self match: each) ifTrue: [
			advance ifFalse: [ self position: start ].
			^each
		].
	].
	^nil!

period
	" Private - Returns the period token. "

	^$.!

position
	"Returns the current position."

	^source position!

position: index
	"Sets the current position."

	^source position: index!

primaryExpression: aContext
	" Private - Parse the receiver looking for a primary expression using variables in aContext.
	Returns nil if failed.
	<rule> primary = variableName | literal | block | '(' expression ')'.
	"

	| node |
	node := self nextLiteral: aContext.
	node notNil ifTrue: [ ^node ].
	node := self nextVariable: aContext.
	node notNil ifTrue: [ ^node ].
"
	(self match: self leftBrace) ifTrue: [
		^self nextQualifiedNameLiteral: aContext
	].
"
	(self match: self leftBracket) ifTrue: [
		^self nextBlock: aContext
	].

	(self match: self leftParenthesis) ifTrue: [
		node := self expression: aContext.
		(self match: self rightParenthesis) ifTrue: [ ^node ].
		^self expected: self rightParenthesis printString
	].
	^nil " self expected: 'primary expression' "!

pushNil
	"Returns a node returning nil."

	^self specialVariableNamed: 'nil'!

pushSelf
	"Returns a node returning self."

	^self specialVariableNamed: 'self'!

readIntegerFrom: aStream radix: radix
	" Private - Answer aNumber of the concrete subclasses if Integer. 
	Initial minus sign accepted, and bases > 10 use letters A-Z.
	Imbedded radix specifiers not allowed;  use Integer 
	class readFrom: for that. Answer zero if there are no digits."

	| digit value |
	value := 0.
	[aStream atEnd] whileFalse: [
		digit := aStream next digitValue.
		(digit < 0 or: [digit >= radix])
		ifTrue: [
			aStream skip: -1.
			^value
		]
		ifFalse: [ value := value * radix + digit ]
	].
	^value!

readSmalltalkFloat: integerPart from: aStream
	" Private -Answer aNumber as described on the stream, aStream. "

	| value coercionClass possibleCoercionClass precision digit num den exp neg endOfNumber eChar atEnd |
	precision := 0.
	num := 0.
	den := 1.
	coercionClass := Integer.
	(aStream peekFor: $.) ifTrue: ["<integer>.<fraction>"
			(aStream atEnd not and: [aStream peek isDigit])
				ifTrue: [
					coercionClass := Float.
					[(atEnd := aStream atEnd) or: [(digit := aStream next) isDigit not]]
						whileFalse: 
							[num := num * 10 + digit digitValue.
							precision := precision + 1].
					den := 10 raisedTo: precision.
					atEnd ifFalse: [aStream skip: -1]]
				ifFalse:
					["oops - just <integer>."
					"un-gobble the period"
					aStream skip: -1]].
	eChar := aStream peek.
	eChar == nil
		ifTrue: [possibleCoercionClass := nil]
		ifFalse:
			[true"SystemUtils isVW20LanguageCompatible" ifTrue: [eChar := eChar asLowercase].
			possibleCoercionClass := self chooseFloatRepresentationFor: eChar.
			possibleCoercionClass == nil ifFalse: [aStream next]].
	exp := nil.
	possibleCoercionClass == nil
		ifFalse:
			[coercionClass := possibleCoercionClass.
			endOfNumber := aStream position.
			neg := aStream peekFor: $-.
			((digit := aStream peek) notNil and: [digit isDigit])
				ifTrue:
					[exp := self readIntegerFrom: aStream radix: 10.
					neg ifTrue: [exp := exp negated]]
				ifFalse: [aStream position: endOfNumber]].
	value := integerPart + (num / den). "The exponent will be added in the next step."
	" ^self
		coerce: value
		to: coercionClass
		precision: precision
		exponent: exp
		exponentChar: eChar
	"
	^exp isNil ifTrue: [ value ]
		ifFalse: [ value * (10 raisedTo: exp) ]!

readSmalltalkNumberFrom: aStream 
	" Private - Answer aNumber as described on the stream, aStream."

	| value neg radix |
	(aStream atEnd or: [aStream peek isLetter]) ifTrue: [^0].
	neg := aStream peekFor: $-.
	value := self readIntegerFrom: aStream radix: 10.
	(aStream peekFor: $r) ifTrue: [
		radix := value.
		value := self readSmalltalkRadix: value from: aStream.
		value := self checkForOldSyntax: radix with: value on: aStream
	]
	ifFalse: [
		value := self readSmalltalkFloat: value from: aStream
	].
	^neg
		ifTrue: [value negated]
		ifFalse: [value]!

readSmalltalkRadix: radix from: aStream
	" Private - Answer aNumber as described on the stream, aStream."

	radix < 2 ifTrue: [ ^self syntaxError: 'Invalid radix' ].
	(aStream atEnd or: [(aStream peek digitValue between: 0 and: radix - 1) not])
	ifTrue: [ aStream skip: -1. ^radix ].
	^self readIntegerFrom: aStream radix: radix!

receiver: anObject
	"The receiver will parse source considering the receiver during evaluation will be of anObject species. "

	| species allInstVarNames index |
	species := anObject species.
	allInstVarNames := species allInstVarNames.
	index := 0.
	self instanceVariables: (allInstVarNames
		collect: [:each|
			index := index + 1.
			ParseInstanceVariable name: each index: index.
		])!

rightBrace
	" Private - Returns the rightBrace token. "

	^$}!

rightBracket
	" Private - Returns the rightBracket token. "

	^$]!

rightParenthesis
	" Private - Returns the rightParenthesis token. "

	^$)!

scope
	"Returns the receiver's root scope."

	scope isNil ifTrue: [ self initializeScope ].
	^scope!

scope: aScope
	"Sets the receiver's root scope."

	scope := aScope!

semicolon
	" Private - Returns the semicolon token. "

	^$;!

singleQuote
	" Private - Returns the singleQuote token. "

	^$'!

skip: aCharacter
	"Skip aCharacter from source stream if present."

	self peek = aCharacter ifTrue: [ self next ].!

skipSeparators
	"Advance to the source stream to the next non-separator character."

	[
	source atEnd ifTrue: [ ^self ].
	self basicPeek isSeparator
	] whileTrue: [ source next ]!

specialVariableNamed: aName
	" Private - Returns a variable known by the receiver with name aName. "

	| index |
	aName = 'self' ifTrue: [ ^ParseSelf new ].
	aName = 'nil' ifTrue: [ ^ParseLiteral value: nil ].
	aName = 'true' ifTrue: [ ^ParseLiteral value: true ].
	aName = 'false' ifTrue: [ ^ParseLiteral value: false ].
	self instanceVariables do: [:each|
		each name = aName ifTrue: [ ^each ].
	].
	^nil "variable not known"!

startOfNextToken
	"Advance to next token and return current position."

	self skipSeparators.
	^self position!

statements: aContext innerBlock: isInnerBlock
	" Private - Parse the next statements in the receiver looking for variables in aContext. "

	| statements returns more expression start origin |
	origin := self startOfNextToken.
	statements := ParseStatements new.
	returns := false.
	more := true.
	[more] whileTrue: [
		start := self startOfNextToken.
		returns := self match: self upArrow.
		returns ifTrue: [ "Return expression."
			expression := self expression: aContext.
			expression isNil ifTrue: [ ^self expected: 'Expression to return' ].
			statements addLast: (
					ReturnNode new
						expression: expression;
						sourceRange: (start to: self position);
						yourself
				).
		]
		ifFalse: [
			expression := self expression: aContext.
			statements addLast: (
				expression isNil ifFalse: [ expression ]
				ifTrue: [
					isInnerBlock
						ifTrue: [ self pushNil ]
						ifFalse: [ self pushSelf ]
				])
		].
		returns ifTrue: [ self skip: self period ].
		more := returns not and: [ self match: self period ]
	].
	self skipSeparators.
	self atEnd ifFalse: [ self expected: 'Message selector or period' ].
	statements sourceRange: (origin to: self position).
	^statements!

statementsArgs: argNodes temps: oldTemps in: aContext
	" Private - Parse the receiver looking for a statements in a block using variables in aContext.
	Returns nil if failed.
	<node> $[ ($: var)* temporaries statements $] => BlockNode
	oldTemps is nil if temps should be parsed.
	This is for compatibility with the old language,	so that temps can be parsed before the	primitive specification.
	"

	| tempNodes stmts start blockComment returnStart parseNode |
	stmts := BlockNode new.
	stmts
		temporaries: oldTemps;
		arguments: argNodes.
	"give initial comment to block, since others trail statements"
	start := self position.
	[
		(self match: self upArrow)
		ifTrue: [
			returnStart := self position.
			parseNode := self expression: aContext.
			parseNode isNil ifTrue: [ ^self expected: 'Expression to return' ].
			parseNode := ReturnNode new
				expression: parseNode;
				sourcePosition: (returnStart to: self position);
				yourself.
			stmts addLast: parseNode.
			self match: self period.  "allow optional trailing . after ^"
			false
		]
		ifFalse: [
			parseNode := self expression: aContext.
			parseNode notNil ifTrue: [
				stmts addLast: parseNode.
				self match: self period "returns true when a new expression is expected"
			]
			ifFalse: [false]
		]
	] whileTrue: [].
	stmts sourcePosition: (start to: self position).
	^stmts!

syntaxError: aString
	"A Syntax error has been occurred."

	| syntaxError |
	syntaxError := SyntaxError description: aString.
	syntaxError
		source: source contents;
		position: self position + 1;
		yourself.
	^errorBlock value: syntaxError!

temporaries: aContext
	" Private - Read temporaries from the receiver's stream.
	<rule> temporaries = [ '|' (variable) ]  (variable, ..., variable)*
	"

	| vars word start temporary |
	(self match: self verticalBar) ifTrue: [ ^#() ].	"no temps"
	vars := OrderedCollection new.
	[	start := self startOfNextToken.
		word := self nextWord.
		word notNil ] whileTrue: [
		self position: start.
		temporary := self nextArgument: aContext.
		temporary canBeAssigned: true.
		vars addLast: temporary.
	].
	(self match: self verticalBar) ifTrue: [ ^vars ].
	^self expected: 'Vertical bar'!

upArrow
	" Private - Returns the upArrow token. "

	^$^!

verticalBar
	" Private - Returns the verticalBar token. "

	^$|! !

ScriptParser categoriesFor: #'arrayLiteralElement' are: #('scanning arrays')!
ScriptParser categoriesFor: #'assignment:in:' are: #('parsing')!
ScriptParser categoriesFor: #'assignment:in:startPosition:' are: #('parsing')!
ScriptParser categoriesFor: #'atEnd' are: #('scanning')!
ScriptParser categoriesFor: #'basicPeek' are: #('scanning')!
ScriptParser categoriesFor: #'braceExpression:' are: #('parsing')!
ScriptParser categoriesFor: #'cascade:to:in:' are: #('parsing')!
ScriptParser categoriesFor: #'checkForOldSyntax:with:on:' are: #('scanning numbers')!
ScriptParser categoriesFor: #'chooseFloatRepresentationFor:' are: #('scanning numbers')!
ScriptParser categoriesFor: #'colon' are: #('tokens')!
ScriptParser categoriesFor: #'constant' are: #('scanning')!
ScriptParser categoriesFor: #'dollar' are: #('tokens')!
ScriptParser categoriesFor: #'expected:' are: #('error management')!
ScriptParser categoriesFor: #'expression:' are: #('parsing')!
ScriptParser categoriesFor: #'initializeInstanceVariables' are: #('initialization')!
ScriptParser categoriesFor: #'initializeScope' are: #('initialization')!
ScriptParser categoriesFor: #'instanceVariables' are: #('accessing')!
ScriptParser categoriesFor: #'instanceVariables:' are: #('accessing')!
ScriptParser categoriesFor: #'isBinarySelector:' are: #('scanning')!
ScriptParser categoriesFor: #'isKeywordSelector:' are: #('scanning')!
ScriptParser categoriesFor: #'isSelectorCharacter:' are: #('tokens')!
ScriptParser categoriesFor: #'isUnarySelector:' are: #('scanning')!
ScriptParser categoriesFor: #'leftArrow' are: #('tokens')!
ScriptParser categoriesFor: #'leftBrace' are: #('tokens')!
ScriptParser categoriesFor: #'leftBracket' are: #('tokens')!
ScriptParser categoriesFor: #'leftParenthesis' are: #('tokens')!
ScriptParser categoriesFor: #'literalQuote' are: #('tokens')!
ScriptParser categoriesFor: #'match:' are: #('scanning')!
ScriptParser categoriesFor: #'message1:repeat:in:' are: #('parsing')!
ScriptParser categoriesFor: #'message2:repeat:in:' are: #('parsing')!
ScriptParser categoriesFor: #'message3:repeat:in:' are: #('parsing')!
ScriptParser categoriesFor: #'methodScope' are: #('parsing')!
ScriptParser categoriesFor: #'mustHaveA:' are: #('scanning')!
ScriptParser categoriesFor: #'newSelectorName:' are: #('parsing')!
ScriptParser categoriesFor: #'next' are: #('scanning')!
ScriptParser categoriesFor: #'nextArgument:' are: #('parsing')!
ScriptParser categoriesFor: #'nextArray' are: #('scanning')!
ScriptParser categoriesFor: #'nextArrayConstant:' are: #('parsing')!
ScriptParser categoriesFor: #'nextBlock:' are: #('parsing')!
ScriptParser categoriesFor: #'nextByteArray' are: #('scanning')!
ScriptParser categoriesFor: #'nextCharacter' are: #('scanning characters')!
ScriptParser categoriesFor: #'nextCharacterConstant:' are: #('parsing')!
ScriptParser categoriesFor: #'nextLiteral:' are: #('parsing')!
ScriptParser categoriesFor: #'nextNumber' are: #('scanning')!
ScriptParser categoriesFor: #'nextNumberConstant:' are: #('parsing')!
ScriptParser categoriesFor: #'nextSelector' are: #('scanning')!
ScriptParser categoriesFor: #'nextSelectorOrNil' are: #('scanning')!
ScriptParser categoriesFor: #'nextString' are: #('scanning strings')!
ScriptParser categoriesFor: #'nextStringConstant:' are: #('parsing')!
ScriptParser categoriesFor: #'nextSymbolConstant:' are: #('parsing')!
ScriptParser categoriesFor: #'nextVariable:' are: #('parsing')!
ScriptParser categoriesFor: #'nextWord' are: #('scanning')!
ScriptParser categoriesFor: #'notFullyImplemented:' are: #('error management')!
ScriptParser categoriesFor: #'parse:onError:' are: #('- public services')!
ScriptParser categoriesFor: #'peek' are: #('scanning')!
ScriptParser categoriesFor: #'peekAny:' are: #('scanning')!
ScriptParser categoriesFor: #'peekAny:advance:' are: #('scanning')!
ScriptParser categoriesFor: #'period' are: #('tokens')!
ScriptParser categoriesFor: #'position' are: #('scanning')!
ScriptParser categoriesFor: #'position:' are: #('scanning')!
ScriptParser categoriesFor: #'primaryExpression:' are: #('parsing')!
ScriptParser categoriesFor: #'pushNil' are: #('node encoding')!
ScriptParser categoriesFor: #'pushSelf' are: #('node encoding')!
ScriptParser categoriesFor: #'readIntegerFrom:radix:' are: #('scanning numbers')!
ScriptParser categoriesFor: #'readSmalltalkFloat:from:' are: #('scanning numbers')!
ScriptParser categoriesFor: #'readSmalltalkNumberFrom:' are: #('scanning numbers')!
ScriptParser categoriesFor: #'readSmalltalkRadix:from:' are: #('scanning numbers')!
ScriptParser categoriesFor: #'receiver:' are: #('accessing')!
ScriptParser categoriesFor: #'rightBrace' are: #('tokens')!
ScriptParser categoriesFor: #'rightBracket' are: #('tokens')!
ScriptParser categoriesFor: #'rightParenthesis' are: #('tokens')!
ScriptParser categoriesFor: #'scope' are: #('accessing')!
ScriptParser categoriesFor: #'scope:' are: #('accessing')!
ScriptParser categoriesFor: #'semicolon' are: #('tokens')!
ScriptParser categoriesFor: #'singleQuote' are: #('tokens')!
ScriptParser categoriesFor: #'skip:' are: #('scanning')!
ScriptParser categoriesFor: #'skipSeparators' are: #('scanning')!
ScriptParser categoriesFor: #'specialVariableNamed:' are: #('parsing')!
ScriptParser categoriesFor: #'startOfNextToken' are: #('scanning')!
ScriptParser categoriesFor: #'statements:innerBlock:' are: #('parsing')!
ScriptParser categoriesFor: #'statementsArgs:temps:in:' are: #('parsing')!
ScriptParser categoriesFor: #'syntaxError:' are: #('error management')!
ScriptParser categoriesFor: #'temporaries:' are: #('parsing')!
ScriptParser categoriesFor: #'upArrow' are: #('tokens')!
ScriptParser categoriesFor: #'verticalBar' are: #('tokens')!

!ServiceError class publicMethods !

description: aString
	"Returns an instance of the receiver with description aString."

	^self new
		description: aString;
		yourself! !

ServiceError class categoriesFor: #'description:' are: #('instance creation')!

!ServiceError publicMethods !

description
	"Returns the receivers description string."

	description isNil ifTrue: [ self initializeDescription ].
	^description!

description: aText
	"Sets the receivers description string."

	description := aText!

descriptionString
	"Returns the receivers description as shown to the user."

	^self description!

initializeDescription
	" Private - Initializes the receiver's description. "

	description := '*Error desconocido*'.!

printOn: aStream
	"Append to the argument aStream, a sequence of characters that describes the receiver."

	aStream
		nextPutAll: self class name,': ';
		nextPutAll: self descriptionString! !

ServiceError categoriesFor: #'description' are: #('accessors')!
ServiceError categoriesFor: #'description:' are: #('accessors')!
ServiceError categoriesFor: #'descriptionString' are: #('accessors')!
ServiceError categoriesFor: #'initializeDescription' are: #('initialization')!
ServiceError categoriesFor: #'printOn:' are: #('CLDT')!

!SyntaxError publicMethods !

initializeDescription
	" Private - Initializes the receiver's description. "

	description := 'Syntax Error'.! !

SyntaxError categoriesFor: #'initializeDescription' are: #('initialization')!

ParseBlock initializeAfterLoad!
ParseContext initializeAfterLoad!
ParseNode initializeAfterLoad!
AssignNode initializeAfterLoad!
MessageNode initializeAfterLoad!
ParseStatements initializeAfterLoad!
BlockNode initializeAfterLoad!
CascadeNode initializeAfterLoad!
Script initializeAfterLoad!
ParseVariable initializeAfterLoad!
ParseArgument initializeAfterLoad!
ParseGlobalVariable initializeAfterLoad!
ParseGlobalConstant initializeAfterLoad!
ParseLiteral initializeAfterLoad!
ParseSpecialVariable initializeAfterLoad!
ParseInstanceVariable initializeAfterLoad!
ParseSelf initializeAfterLoad!
ReturnNode initializeAfterLoad!
ScriptParser initializeAfterLoad!
ServiceError initializeAfterLoad!
ParseError initializeAfterLoad!
SyntaxError initializeAfterLoad!
Scripting initializeAfterLoad!

Scripting loaded!